using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using System.IO;
using System.Diagnostics;
using System.Reflection;
using System.Threading;

using MT.FPSource;

using DShowNET;
using DShowNET.Device;

namespace GraphController
{
	/// <summary>
	/// Zusammenfassung fr clsFilterGraphController.
	/// </summary>
	public class FilterGraphControllerSource : FilterGraphController
	{
		// --------------------- Events ----------------------

		// ------------- Constructors/Destructors --------------

		protected string			_sourceFile = null;
		
		private IntPtr				_drainWin;
		private	bool				_fullScreen;
		private bool			    _strg;	

		/// <summary> type of clip, video / audio. </summary>
		private	ClipType				_clipType;
		private	PlayState				_playState;
		protected System.Windows.Forms.UserControl			_Window = null;				// Property Backer: Owner control for preview

		private const int WM_GRAPHNOTIFY	= 0x00008001;	// message from graph

		public enum PlayState
		{
			None, Init, Stopped, Paused, Running, Error
		}

		public enum ClipType
		{
			None, AudioVideo, VideoOnly, AudioOnly
		}
		/// <summary> 
		///  Create a new Capture object. 
		///  _videoDevice and _audioDevice can be null if you do not 
		///  wish to capture both audio and video. However at least
		///  one must be a valid device. Use the <see cref="Filters"/> 
		///  class to list available devices.
		///  </summary>


		public FilterGraphControllerSource(string sourceFile, Filter audioDevice)
		{			
			if ( sourceFile == null)
				throw new ArgumentException( "The _sourceFile parameter must be set to a valid File.\n" );
			this._videoDevice = null;
			this._sourceFile = sourceFile;
			this._audioDevice = audioDevice;
			this._filename = getTempFilename();
			createGraph();

			_grabVideo = new FPDetector(_sampGrabber);
			_grabImage = new SampleGrabber(_sampGrabber);

			_grabImage.BeginCapture +=new GraphController.SampleGrabber.EmptyDelegate(_grabImage_BeginCapture);
			_grabImage.EndCapture += new GraphController.SampleGrabber.EmptyDelegate(_grabImage_EndCapture);

			//Deactivate SWFilter
			//Set/Get Parameters of the Interface
			SWFilterParams	parset;

			//Deactivate Networkcommunication of SWFilter
			int hr = _swInterface.GetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
					
			parset.intNet = 0;

			hr = _swInterface.SetParams(out parset);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
		}


 		/// <summary> Destructor. Dispose of resources. </summary>
		~FilterGraphControllerSource()
		{
			Dispose();
		}

	
		public System.Windows.Forms.UserControl	USC
		{
			set
			{
				_Window = value;
			}
		}
		public string SourceFile
		{
			get{return(_sourceFile);}
			set
			{
				_sourceFile = value;
			}
		}

		public override Control PreviewWindow
		{
			get { return( _previewWindow ); }
			set
			{
				assertStopped();
				derenderGraph();
				_previewWindow = value;
				_wantPreviewRendered = ( ( _previewWindow != null ) && ( _sourceFile != null ) );
				renderGraph();
				startPreviewIfNeeded();
			}
		}

		/// <summary> build the capture graph. </summary>
		protected override bool SetupGraph(GraphState graphState)
		{
			int hr;
			int nextPin;
			IPin					pinOut;
			IPin					pinIn;
			IPin					pinOutRender;

			try 
			{
				if((int)graphState < (int)GraphState.Created)
				{
					hr = _graphBuilder.AddSourceFilter(_sourceFile,_sourceFile, out _sourceFilter);
				}
					
	
				if(graphState != GraphState.Capturing)
				{

					hr= DsUtils.GetPin(_sourceFilter,PinDirection.Output  ,0, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);
	
					hr = DsUtils.GetPin(_aviSplitter,PinDirection.Input ,0,out pinIn);

					hr = _graphBuilder.AddFilter( _aviSplitter, "AVI Splitter" );
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					hr = _graphBuilder.Connect(pinOut, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

			
					hr= DsUtils.GetPin(_aviSplitter,PinDirection.Output,0, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

						

					hr = DsUtils.NextPin(_aviSplitter,PinDirection.Output,out nextPin);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_aviSplitter,PinDirection.Output,nextPin-1, out pinOutRender);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.Render(pinOutRender);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);



					hr = _graphBuilder.AddFilter( _infTee, "Ds.NET Infinite Pin Tee Filter" );
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					hr= DsUtils.GetPin(_infTee,PinDirection.Input ,0, out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.Connect(pinOut, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					//***********************************************************
					//  SampleGrabber Filter
					//***********************************************************
				
					hr = DsUtils.NextPin(_infTee,PinDirection.Output,out nextPin);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_infTee,PinDirection.Output,nextPin-1, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);
				
					hr = _graphBuilder.AddFilter(_nullRenderer,"Null Renderer");
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					//Add SampleGrabber Filter 
					hr = _graphBuilder.AddFilter(_baseGrabFlt, "Ds.NET Grabber");
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );			
					//Get Input Pin from SampleGrabber Filter
					hr= DsUtils.GetPin(_baseGrabFlt, PinDirection.Input,0, out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);	
			
					hr = _graphBuilder.Connect(pinOut, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );	
				
					hr= DsUtils.GetPin(_baseGrabFlt, PinDirection.Output,0, out pinOutRender);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);
				
					hr= DsUtils.GetPin(_nullRenderer, PinDirection.Input ,0,out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.Connect(pinOutRender, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );
					//*************************************************
					// END SampleGrabber Filter
					//*************************************************

					//***********************************************************
					//  Transform Filter
					//***********************************************************
					hr = DsUtils.NextPin(_infTee,PinDirection.Output,out nextPin);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_infTee,PinDirection.Output,nextPin-1, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.AddFilter(_nullRenderer2,"Null Renderer");
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					//Add Transform Filter
					hr = _graphBuilder.AddFilter(_transformFilter,"SWFilter");
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					//Get Input Pin from Transform Filter
					hr= DsUtils.GetPin(_transformFilter, PinDirection.Input,0, out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);		
								
					hr = _graphBuilder.Connect(pinOut, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );

					hr= DsUtils.GetPin(_transformFilter, PinDirection.Output,0,out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_nullRenderer2, PinDirection.Input ,0,out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.Connect(pinOut, pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );
				}
				//*************************************************
				// END Transform Filter
				//*************************************************


				Console.WriteLine("Aktueller Aufbau des Filter Graphs:");
				hr = DsUtils.EnumFilters(_filterGraph);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );


				Guid time = TimeFormat.Frame;
				
				hr = _mediaSeek.SetTimeFormat(ref time);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );	

				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine("Could not setup graph\r\n" + ee.Message );
				return false;
			}
			finally
			{

			}
		}

		protected override void createGraph()
		{
			Guid					cat;
			Guid					med;
			int						hr;
			
			
			// Ensure required properties are set
			if ( _videoDevice == null)
				Console.WriteLine( "The video device have not been set. Please set one or both to valid capture devices.\n" );

			// Skip if we are already created
			if ( (int)_graphState < (int)GraphState.Created )
			{
				// Garbage collect, ensure that previous filters are released
				GC.Collect();
				
				GetInterfaces();
				GetFrameStepInterface();
				


				// Get the video device and add it to the filter graph
				if ( VideoDevice != null )
				{
					_devFilter = (IBaseFilter) Marshal.BindToMoniker( VideoDevice.MonikerString );
				}

				// Add the graph to the Running Object Table so it can be
				// viewed with GraphEdit
				SetupGraph(_graphState);

				#if DEBUG
					DsROT.AddGraphToRot( _graphBuilder, out _rotCookie );
				#endif
				// Retrieve the stream control interface for the video device
				// FindInterface will also add any required filters
				// (WDM devices in particular may need additional
				// upstream filters to function).

				// Try looking for an interleaved media type
				object comObj = null;
				cat  = PinCategory.Capture;
				med = MediaType.Interleaved;
				Guid iid = typeof(IAMStreamConfig).GUID;

				hr = _capGraph.FindInterface( ref cat, ref med,_devFilter, ref iid, out comObj );
				if( hr != 0 )
				{
					// If not found, try looking for a video media type
					med = MediaType.Video;
					hr = _capGraph.FindInterface( ref cat, ref med, _devFilter, ref iid, out comObj );
				
					if( hr != 0 )
						comObj= null;
				}
		
				//Instead of calling OleCreatePropertyFrame do ...
				_videoStreamConfig=comObj as IAMStreamConfig;

				if(_videoStreamConfig == null) 
				{
					Console.WriteLine("IAMStreamConfig not present");
				}


				// Reload any video crossbars
				if ( _videoSources != null ) _videoSources.Dispose(); _videoSources = null;

				// Reload any property pages exposed by filters
				if ( _propertyPages != null ) _propertyPages.Dispose(); _propertyPages = null;

				// Reload capabilities of video device
				_videoCaps = null;

				_graphState = GraphState.Created;
			}
		}

		/// <summary>
		///  Disconnect and remove all filters except the device
		///  and compressor filters. This is the opposite of
		///  renderGraph(). Soem properties such as FrameRate
		///  can only be set when the device output pins are not
		///  connected. 
		/// </summary>
		protected override void derenderGraph()
		{
			// Stop the graph if it is running (ignore errors)
			if ( _mediaCtrl != null )
				_mediaCtrl.Stop();

			// Free the preview window (ignore errors)
			if ( _videoWin != null )
			{
				_videoWin.put_Visible( DsHlp.OAFALSE );
				_videoWin.put_Owner( IntPtr.Zero );
				_videoWin = null;
			}

			// Remove the Resize event handler
			if ( _previewWindow != null )
				_previewWindow.Resize -= new EventHandler( onPreviewWindowResize );

			if ( (int)_graphState >= (int)GraphState.Rendered )
			{
				// Update the state
				_graphState = GraphState.Created;
				_isCaptureRendered = false;
				_isPreviewRendered = false;

				// Disconnect all filters downstream of the 
				// video and audio devices. If we have a compressor
				// then disconnect it, but don't remove it
				//				if ( _infTee != null )
				//					removeDownstream( _infTee, (_videoCompressor==null) );
				if ( _sourceFilter != null )
				{
					removeDownstream( _sourceFilter, (_videoCompressor==null) );
					
					if(!_wantCapture)
					{
						Type comtype = null;
						object comobj = null;
						// Get a new Instance of InfTee when Resolution or Framerate changed
						_infTee = null;
						// Get the Infinite Tee Pin Filter
						comtype = Type.GetTypeFromCLSID( Clsid.InfTee);
						if( comtype == null )
							throw new NotSupportedException( "DirectX (8.1 or higher) not installed?" );
						comobj = Activator.CreateInstance(comtype);
						Console.WriteLine("Infinite Pin Tee Filter wurde erzeugt");
						_infTee = (IBaseFilter) comobj; comobj = null;
						SetupGraph(_graphState);
						_graphState = GraphState.Rendered;
					}
					else
					{
						_graphState = GraphState.Rendered;
					}
				}

				// These filters should have been removed by the
				// calls above. (Is there anyway to check?)
				_muxFilter = null;
				_fileWriterFilter = null;
			}
		}
		/// <summary>
		///  Connects the filters of a previously created graph 
		///  (created by createGraph()). Once rendered the graph
		///  is ready to be used. This method may also destroy
		///  streams if we have streams we no longer want.
		/// </summary>
		protected override void renderGraph()
		{
			//			Guid					cat;
			//			Guid					med;
			int						hr;
			int						nextPin;
			bool					didSomething = false;
			IPin					pinIn;
			IPin					pinOut;
			const int WS_CHILD			= 0x40000000;	
			const int WS_CLIPCHILDREN	= 0x02000000;
			const int WS_CLIPSIBLINGS	= 0x04000000;

			assertStopped();

			// Ensure required properties set
			if ( _filename == null )
				throw new ArgumentException( "The _filename property has not been set to a file.\n" );

			// Stop the graph
			if ( _mediaCtrl != null )
				_mediaCtrl.Stop();

			// Create the graph if needed (group should already be created)
			createGraph();



			// Derender the graph if we have a capture or preview stream
			// that we no longer want. We can't derender the capture and 
			// preview streams seperately. 
			// Notice the second case will leave a capture stream intact
			// even if we no longer want it. This allows the user that is
			// not using the preview to Stop() and Start() without
			// rerendering the graph.
			if ( !_wantPreviewRendered && _isPreviewRendered )
				derenderGraph();
			if ( !_wantCaptureRendered && _isCaptureRendered )
				if ( _wantPreviewRendered )
					derenderGraph();


			// Render capture stream (only if necessary)
			if ( _wantCaptureRendered && !_isCaptureRendered )
			{
				//				// Render the file writer portion of graph (mux -> file)
				//				Guid mediaSubType = MediaSubType.Avi;
				//				hr = _capGraph.SetOutputFileName( ref mediaSubType, _filename, out _muxFilter, out _fileWriterFilter );
				//				if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );

				// Render video (video -> mux)
				if ( VideoDevice != null )
				{
					//***********************
					// MUX
					//***********************
					hr = DsUtils.NextPin(_infTee,PinDirection.Output,out nextPin);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_infTee,PinDirection.Output,nextPin-1, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);			
			
					Guid sub = MediaSubType.Avi;
					hr = _capGraph.SetOutputFileName( ref sub, _filename, out _muxFilter, out _fileWriterFilter );
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );		

					

					if(VideoCompressor == null )
					{
						hr= DsUtils.GetPin(_muxFilter,PinDirection.Input,0, out pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR(hr);

						hr = _graphBuilder.Connect(pinOut,pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR( hr );
					}
					else
					{
						//_videoCompressorFilter = (IBaseFilter) Marshal.BindToMoniker( VideoCompressor.MonikerString ); 
						hr = _graphBuilder.AddFilter( _videoCompressorFilter, "Video Compressor" );
						if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );
				
						DsUtils.GetPin(_videoCompressorFilter,PinDirection.Input,0,out pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR(hr);

						hr = _graphBuilder.Connect(pinOut,pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR( hr );

						DsUtils.GetPin(_videoCompressorFilter,PinDirection.Output ,0,out pinOut);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR(hr);

						hr= DsUtils.GetPin(_muxFilter,PinDirection.Input,0, out pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR(hr);

						hr = _graphBuilder.Connect(pinOut,pinIn);
						if( hr < 0 )
							Marshal.ThrowExceptionForHR( hr );
					}

					//***********************
					// END MUX
					//***********************

					//					// Try interleaved first, because if the device supports it,
					//					// it's the only way to get audio as well as video
					//					cat = PinCategory.Capture;
					//					med = MediaType.Interleaved;
					//					hr = _capGraph.RenderStream( ref cat, ref med, _devFilter, _videoCompressorFilter, _muxFilter ); 
					//					if( hr < 0 ) 
					//					{
					//						med = MediaType.Video;
					//						hr = _capGraph.RenderStream( ref cat, ref med, _devFilter, _videoCompressorFilter, _muxFilter ); 
					//						if ( hr == -2147220969 ) throw new DeviceInUseException( "Video device", hr );
					//						if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );
					//					}
				}

				// Render audio (audio -> mux)
				if ( AudioDevice != null )
				{

					hr = DsUtils.NextPin(_infTee,PinDirection.Output,out nextPin);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr= DsUtils.GetPin(_infTee,PinDirection.Output,nextPin-1, out pinOut);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					_audioDeviceFilter = (IBaseFilter) Marshal.BindToMoniker( AudioDevice.MonikerString );
					hr = _graphBuilder.AddFilter( _audioDeviceFilter, "Audio Device Filter" );
					if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );
				
					DsUtils.GetPin(_audioDeviceFilter,PinDirection.Input,0,out pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR(hr);

					hr = _graphBuilder.Connect(pinOut,pinIn);
					if( hr < 0 )
						Marshal.ThrowExceptionForHR( hr );
				}

				Console.WriteLine("Aktueller Aufbau des Filter Graphs:");
				hr = DsUtils.EnumFilters(_filterGraph);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				_isCaptureRendered = true;
				didSomething = true;
			}

			// Render preview stream (only if necessary)
			if ( _wantPreviewRendered && !_isPreviewRendered )
			{
				//				// Render preview (video -> renderer)
				//				cat = PinCategory.Preview;
				//				med = MediaType.Video;
				//				hr = _capGraph.RenderStream( ref cat, ref med, _devFilter, null, null ); 
				//				if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );

				//*************************************************
				// Preview Filter
				//*************************************************
				hr = DsUtils.NextPin(_infTee,PinDirection.Output,out nextPin);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);
				hr= DsUtils.GetPin(_infTee,PinDirection.Output,nextPin-1, out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);
			
				hr =_graphBuilder.Render(pinOut);
				if( hr < 0)
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END Preview Filter
				//*************************************************
				Console.WriteLine("Aktueller Aufbau des Filter Graphs:");
				hr = DsUtils.EnumFilters(_filterGraph);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				hr = _mediaEvt.SetNotifyWindow( _Window.Handle , WM_GRAPHNOTIFY, IntPtr.Zero );
				// Get the IVideoWindow interface
				_videoWin = (IVideoWindow) _graphBuilder;

				// Set the video window to be a child of the main window
				hr = _videoWin.put_Owner( _previewWindow.Handle );
				if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );

				// Set video window style
				hr = _videoWin.put_WindowStyle( WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
				if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );

				// Use helper function to position video window in client rect of owner window
				//resizeVideoWindow();

				// Position video window in client rect of owner window
				_previewWindow.Resize += new EventHandler( onPreviewWindowResize );
				onPreviewWindowResize( this, null );

				// Make the video window visible, now that it is properly positioned
				hr = _videoWin.put_Visible( DsHlp.OATRUE );
				if( hr < 0 ) Marshal.ThrowExceptionForHR( hr );

				_isPreviewRendered = true;
				didSomething = true;
			}

			if ( didSomething )
				_graphState = GraphState.Rendered;
		}


		/// <summary> graph event (WM_GRAPHNOTIFY) handler. </summary>
		public override void OnGraphNotify()
		{
			if( _mediaEvt != null )
			{
				DsEvCode	code;
				int p1, p2, hr = 0;
				do
				{
					hr = _mediaEvt.GetEvent( out code, out p1, out p2, 0 );
					if( hr < 0 )
						break;
					hr = _mediaEvt.FreeEventParams( code, p1, p2 );
					if( code == DsEvCode.Complete)
						OnClipCompleted();
				}
				while( hr == 0 );
			}
			
		}

		//Functions to Controll a Video Clip
		public void ClipPause()
		{
			if( _mediaCtrl == null )
				return;
			
			int hr = _mediaCtrl.Pause();
			if( hr >= 0 )
			{
				_playState = PlayState.Paused;
			}
			else
			{
				return;
			}
		
		}
		
		public void ClipStop()
		{
			if( (_mediaCtrl == null) || (_mediaSeek == null) )
				return;

//			if( (playState != PlayState.Paused) && (playState != PlayState.Running) )
//				return;

			int hr = _mediaCtrl.Stop();
			_playState = PlayState.Stopped;

			DsOptInt64 pos = new DsOptInt64( 0 );
			hr = _mediaSeek.SetPositions( pos, SeekingFlags.AbsolutePositioning, null, SeekingFlags.NoPositioning );
			hr = _mediaCtrl.Pause();
		}

		public void ClipClose(System.Windows.Forms.Form myWindow)
		{
			if( _mediaCtrl == null )
			{
				return;
			}
			_clipType = ClipType.None;
			CloseInterfaces();
			myWindow.Refresh();
		}

		public long ClipLength()
		{
			long d;
			if( (_mediaCtrl == null) || (_mediaSeek == null) )
				return -1;
			
			if( (_playState == PlayState.None))
				return -1;
			
			int hr = _mediaSeek.GetDuration(out d);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
				
			return d;
						
		}

		public int GetPosition()
		{
			long p;
			if( (_mediaCtrl == null) || (_mediaSeek == null) )
				return -1;
			
			if( (_playState == PlayState.None))
				return -1;
			
			int hr = _mediaSeek.GetCurrentPosition(out p);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
				
			return (int)p;
		}
			
		public bool ClipPlay()
		{
			if( _mediaCtrl == null )
				return false;
			
			int hr = _mediaCtrl.Run();
			if( hr >= 0 )
			{
				_playState = PlayState.Running;
			}
			else
			{
				playState = PlayState.Error;
			}


			return hr >= 0;
		}

		/// <summary> step n-numbers of frames in stream. </summary>
		public void StepFrames( int frames )
		{
			int hr;
			
			if( (_videoStep == null) || (_mediaCtrl == null) )
			{
				Console.WriteLine("return mediaCtrl == null");
				return;
			}
			hr = _videoStep.CanStep( frames, null );
			if( hr != 0 )
			{
				Console.WriteLine("return Can't Step");
				return;
			}
			if( _playState != PlayState.Paused )
			{
				hr = _mediaCtrl.Pause();
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
			}

			_playState = PlayState.Paused;
			//myActivities.Items.Add("StepFrames(1)");
			hr = _videoStep.Step( frames, null );
			if( hr != 0 )
			{
				Console.WriteLine("return = StepFailure");
				return;
			}
			

		}

		/// <summary> graph event if clip has finished </summary>
		void OnClipCompleted()
		{
			if( (_mediaCtrl == null) || (_mediaSeek == null) )
				return;

			DsOptInt64 pos = new DsOptInt64( 0 );
			int hr = _mediaSeek.SetPositions( pos, SeekingFlags.AbsolutePositioning, null, SeekingFlags.NoPositioning );
			if( hr == 0 )
				return;
			hr = _mediaCtrl.Stop();
			if( hr < 0 )
				return;
			hr = _mediaCtrl.Run();
		}

		/// <summary> try to detect clip type (video/audio) [not reliable]. </summary>
		public void CheckClipType()
		{
			if( _basicAudio == null )
				_clipType = ClipType.None;
			else
				_clipType = ClipType.AudioOnly;

			if( (_videoWin == null) || (_basicVideo == null) )
				return;

			int visible;
			int hr = _videoWin.get_Visible( out visible );
			if( hr < 0 )
				return;
			else
			{
				if( _basicAudio == null )
					_clipType = ClipType.VideoOnly;
				else
					_clipType = ClipType.AudioVideo;
			}
		}

		public PlayState playState
		{
			get{return _playState;}
			set
			{
				_playState = value;
			}
		}

		public ClipType clipType
		{
			get{return _clipType;}
			set
			{
				_clipType = value;
			}
		}

		
		/// <summary> try to get the step interfaces. </summary>
		bool GetFrameStepInterface()
		{
			_videoStep = _graphBuilder as IVideoFrameStep;
			if( _videoStep == null )
				return false;

			// Check if this decoder can step
			int hr = _videoStep.CanStep( 0, null );
			if( hr != 0 )
			{
				//_videoStep = null;
				return false;
			}
			return true;
		}

		/// <summary> do cleanup and release DirectShow. </summary>
		public void CloseInterfaces()
		{
			int hr;
			try 
			{
				if( _mediaCtrl != null )
				{
					hr = _mediaCtrl.StopWhenReady();
					_mediaCtrl = null;
				}

				_playState = PlayState.Stopped;

				if( _mediaEvt != null )
				{
					hr = _mediaEvt.SetNotifyWindow( IntPtr.Zero, WM_GRAPHNOTIFY, IntPtr.Zero );
					_mediaEvt = null;
				}

				if( _videoWin != null )
				{
					hr = _videoWin.put_Visible( DsHlp.OAFALSE );
					hr = _videoWin.put_Owner( IntPtr.Zero );
					_videoWin = null;
				}

				_mediaSeek	= null;
				_mediaPos	= null;
				_basicVideo	= null;
				_videoStep	= null;
				_basicAudio	= null;

				if( _graphBuilder != null )
					Marshal.ReleaseComObject( _graphBuilder ); _graphBuilder = null;
				
				if( _sourceFilter != null )
					Marshal.ReleaseComObject( _sourceFilter ); _sourceFilter = null;
							
				_playState = PlayState.None;
			}
			catch( Exception )
			{}
		}
		
		public void ModifyRate( double rateAdjust )
		{
			if( (_mediaPos == null) || (rateAdjust == 0.0) )
				return;
			
			double rate;
			int hr = _mediaPos.get_Rate( out rate );
			if( hr < 0 )
				return;
			rate += rateAdjust;
			hr = _mediaPos.put_Rate( rate );
		}

		/// <summary> full-screen toggle. </summary>
		public void ToggleFullScreen()
		{
			if( _videoWin == null )
				return;
	
			int mode;
			int hr = _videoWin.get_FullScreenMode( out mode );
			if( mode == DsHlp.OAFALSE )
			{
				hr = _videoWin.get_MessageDrain( out _drainWin );
				hr = _videoWin.put_MessageDrain( _previewWindow.Handle );
				mode = DsHlp.OATRUE;
				hr = _videoWin.put_FullScreenMode( mode );
				if( hr >= 0 )
					_fullScreen = true;
			}
			else
			{
				mode = DsHlp.OAFALSE;
				hr = _videoWin.put_FullScreenMode( mode );
				if( hr >= 0 )
					_fullScreen = false;
				hr = _videoWin.put_MessageDrain( _drainWin );
				_previewWindow.BringToFront();
				_previewWindow.Refresh();
			}
		}

		public void SetRate( double newRate )
		{
			if( _mediaPos == null )
				return;
			
			int hr = _mediaPos.put_Rate( newRate );
		}
		
		public void SetTrackBar(int myTrackBar)
		{
			long d , p;
			if( (_mediaCtrl == null) || (_mediaSeek == null) )
				return;

			if( (_playState == PlayState.None))
				return;

			int hr = _mediaSeek.GetDuration(out d);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			//p= d/50 * (long)myTrackBar;
			p= (long)myTrackBar;

			DsOptInt64 pos = new DsOptInt64( p );
			hr = _mediaSeek.SetPositions( pos, SeekingFlags.AbsolutePositioning, null, SeekingFlags.NoPositioning );
//			hr = _mediaCtrl.Pause();
//			_playState = PlayState.Paused;
		}

		public void UpdateTrackBar(TrackBar myTrackBar)
		{
			if (_mediaPos != null)
			{
				long p, d;
				
				int hr = _mediaSeek.GetCurrentPosition(out p);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				hr = _mediaSeek.GetDuration(out d);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

//				e= d/100;
//				pos=p/e;
				//if(pos < 100)
					myTrackBar.Value = (int)p;

			}
			
		}

		public void UpdateStatusBar(StatusBarPanel stbStatus,StatusBarPanel stbTime,StatusBarPanel stbTimeLen, StatusBarPanel stbFrame)
		{
			switch (playState)
			{
				case PlayState.Error  : stbStatus.Text = "Error"; break;
				case PlayState.None	  : stbStatus.Text = "None"; break;
				case PlayState.Init   : stbStatus.Text = "Init"; break;
				case PlayState.Paused : stbStatus.Text = "Paused "; break;
				case PlayState.Running: stbStatus.Text = "Running"; break;
				case PlayState.Stopped: stbStatus.Text = "Stopped"; break;
			}

			if (_mediaPos != null)
			{
				double s;
				long p;
				int frame;
				//				long psource, ptarget;
				//				double framerate;
				

				int hr = _mediaPos.get_Duration(out s);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				int h = (int) s / 3600;
				int m = ((int)s  - (h * 3600)) / 60;
				s = (int)s - (h * 3600 + m * 60);
                
				stbTimeLen.Text = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m,(int) s);
				
				
				hr = _mediaPos.get_CurrentPosition(out s);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				h = (int)s / 3600;
				m = (int)(s  - (h * 3600)) / 60;
				s = (int)s - (h * 3600 + m * 60);
				                

				stbTime.Text = String.Format("{0:D2}:{1:D2}:{2:D2}", h, m, (int) s);
				

//				Guid time = TimeFormat.Frame;
//				Guid media = TimeFormat.Media_Time;
//				Guid format = typeof(TimeFormat).GUID;
				
//				hr = _mediaSeek.GetTimeFormat(out format);
//				if( hr < 0 )
//					Marshal.ThrowExceptionForHR( hr );

				hr = _mediaSeek.GetCurrentPosition(out p);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				frame = (int)p + 1;
				
				stbFrame.Text = String.Format(frame.ToString());
			}
			else
			{
				stbTime.Text = "00:00:00";
				stbTimeLen.Text = "00:00:00";
			}
		}

		public void UpdateToolBar(ToolBarButton btnPlay, ToolBarButton	btnPause, ToolBarButton	btnStop)
		{
			switch (_playState)
			{
				case PlayState.Init   : btnPlay.Enabled = true;
					btnPause.Enabled = false;
					btnStop.Enabled = false;
					break;
                                        
				case PlayState.Paused : btnPlay.Enabled = true;
					btnPause.Enabled = false;
					btnStop.Enabled = true;
					break;
                                        
				case PlayState.Running: btnPlay.Enabled = false;
					btnPause.Enabled = true;
					btnStop.Enabled = true;
					break;
                                        
				case PlayState.Stopped: btnPlay.Enabled = true;
					btnPause.Enabled = false;
					btnStop.Enabled = false;
					break;
			}

		}

		public void UpdateInfo(StatusBarPanel stbStatus,StatusBarPanel stbFrame, StatusBarPanel stbTime, StatusBarPanel stbTimeLen ,TrackBar trackVideo)
		{
			if (_playState == PlayState.Running || _playState == PlayState.Paused)
			{
				UpdateStatusBar(stbStatus,stbTime,stbTimeLen,stbFrame);
				UpdateTrackBar(trackVideo);
			}
		}

		public void SetKeys(KeyEventArgs e)
		{
			if( _playState == PlayState.Init )
				return;

			switch( e.KeyCode )
			{
				case Keys.ControlKey:
				{	_strg = true; break;}

				case Keys.D1:
				{ 
					if (_strg == true)
					{
						ToggleFullScreen();
						_strg = false;
					}
					break;
				}
				case Keys.D2:
				{ 
					if (_strg == true)
					{
						ToggleFullScreen();
						_strg = false;
					}
					break;
				}
				case Keys.D3:
				{ 
					if (_strg == true)
					{
						ToggleFullScreen();
						_strg = false;
					}
					break;
				}
			
				case Keys.Space:
				{	
					StepFrames( 1 );
					break; 
				}

				case Keys.Escape:
				{
					ToggleFullScreen();
					break;
				}

				case Keys.F1:
				{
					if( ! ClipPlay()){}	

					break;
				}
				case Keys.F2:
				{
					ClipPause();

					break;
				}
				case Keys.F3:
				{
					ClipStop();

					break;
				}

			}
		}



	}


}
