using System;
using MT.Tracker.DataStructures;

namespace MT.Tracker
{
	/// <summary>
	/// A Hypothesis object contains a Triplet of Featurepoints from 
	/// 3 consecutive Frames which fullfilled all constraints to be
	/// considered in further tracking analysis.
	/// </summary>
	public class Hypothesis
	{
		FeaturePoint _P1;
		FeaturePoint _P2;
		FeaturePoint _P3;
		float _Cost;
		int	_Index;
		eState _state;

		public enum eState
		{	unverified = -1,
			rejected = 0,
			tested = 1
		}

		public Hypothesis(FeaturePoint P1, FeaturePoint P2, FeaturePoint P3,float Cost)
		{		
			_P1=P1;
			_P2=P2;
			_P3=P3;

			//Debug Code
			if (_P1.next !=null && _P2 != _P1.next)
			{
				Console.WriteLine("Invalid Hypothesis");
			}
			if (_P2.previous !=null && _P1 != _P2.previous)
			{
				Console.WriteLine("Invalid Hypothesis");
			}
			if (_P2.next !=null && _P3 != _P2.next)
			{
				Console.WriteLine("Invalid Hypothesis");
			}
			if (_P3.previous !=null && _P2 != _P3.previous)
			{
				Console.WriteLine("Invalid Hypothesis");
			}
			//End Debug Code
			
			_Index = Index;
			_Cost = Cost;

		}

		/// <summary>
		/// Connects all Featurepoints of the Hypothesis and therefor
		/// creates a Trajectory out of it.
		/// If the first and/or the second FP belong
		/// to an existing Trajectory, then they will 
		/// use the ID of this Trajectory, otherwise
		/// they use the id of the Argument TrajectorID 
		/// increased by one.
		/// </summary>
		/// <param name="TrajectorID"></param>
		/// <returns></returns>
		public	bool	connect() //int TrajectorID
		{
			// x o o
//			if (_P2.tid == -1 &&
//				_P3.tid == -1)
			if (_P1.next == null && _P2.previous == null &&
				_P2.next == null && _P3.previous == null)
			{
				//if (_P1.tid == -1) {_P1.tid =TrajectorID;}
				//				_P2.tid = _P1.tid;
				//				_P3.tid = _P1.tid;
				_P1.next = _P2;
				_P2.previous = _P1;
				_P2.next = _P3;
				_P3.previous = P2;

				return true;
			}
				// x-x o
				//			else if (_P1.tid ==_P2.tid && 
				//					 _P2.next == null &&
				//					 _P3.tid == -1) 
			else if (_P1.next ==_P2 && _P2.previous == _P1 && 
				_P2.next == null && P3.previous == null)
			{
				//if (_P1.tid == -1) {_P1.tid =TrajectorID;}
				//				_P3.tid = _P1.tid;
				_P2.next = _P3;
				_P3.previous = P2;
				
				return true;
			}
				// x y o <- should not happen
			else 
			{
				Console.WriteLine("Invalid Connect Attempt");
			}
//				if (_P1.tid != _P2.tid &&
//				 	 _P1.next == null &&
//					 _P2.previous == null &&
//					 _P2.next == null &&
//					 _P3.tid == -1)
//			{
//				_P1.next = _P2;
//				_P2.previous = P1;
////				_P2.tid = P1.tid;
//				_P2.next = _P3;
//				_P3.previous = _P2;
////				_P3.tid = _P2.tid;
//
//				return true;
//			}

			return false;

		}

		public	double		 Angle
		{
			get 
			{
				vector2f a = new vector2f(_P2.X - _P1.X, 
					_P2.Y - _P1.Y);

				vector2f b = new vector2f(_P3.X - _P2.X, 
					_P3.Y - _P2.Y);

				return Math.Acos(a*b/(a.length*b.length))/2/Math.PI*360;
			}
		}

		public override string ToString()
		{
			return string.Format("H({0}): P({1,2})-P({2,2})-P({3,2}): {4}",_Index,P1.fid,P2.fid,P3.fid,_Cost.ToString("n:0.000000"));
		}


		#region "Properties"

		public	FeaturePoint P1
		{
			get { return _P1; }
		}

		public	FeaturePoint P2
		{
			get { return _P2; }
		}

		public	FeaturePoint P3
		{
			get { return _P3; }
		}

		public	float	Cost
		{	get	{ return _Cost; } 
		}

		public  int		Index
		{
			get { return _Index;  }
			set { _Index = value; }
		}

		public	eState	State
		{
			get	{ return _state;  }
			set { _state = value; }
		}

		#endregion

	}
}




//Legacy Code
//			vector2f a = new vector2f(_P2.X - _P1.X, 
//									  _P2.Y - _P1.Y);
//
//			vector2f b = new vector2f(_P3.X - _P2.X, 
//									  _P3.Y - _P2.Y);
//
//			w1 = (float)w1*a.length/Vmax;
//			w2 = 1- w1;
//
//			//Prevent Zero Values
//			double al = Math.Max(a.length,0.01);
//			double bl = Math.Max(b.length,0.01);
//
//			double alxbl = al * bl;
//
//			_Cost = (float)(w1*(1-Math.Max(a*b/alxbl,0)) + 
//				            w2*(1-2*Math.Sqrt(alxbl)/(al + bl)));

//			vector2f a = new vector2f(P2.X - P1.X, 
//									  P2.Y - P1.Y);
//
//			vector2f b = new vector2f(P3.X - P2.X,
//									  P3.Y - P2.Y);
//
//			w1 = 0.5f;
//			w2 = 0.5f;
//
//			double dx = Math.Abs(a.X - b.X);
//			double dy = Math.Abs(a.Y - b.Y);

//			_Cost =  (float)(w1*dx/Vmax + w2*dy/Vmax);

			//_Cost  = Gamma(_P1,_P2,_P3, Vmax);	


//		public static float Gamma(FeaturePoint P1, FeaturePoint P2, FeaturePoint P3, int Vmax)
//		{
//			float w1 = 0.9f;	//Angleweight
//			float w2 = 0.1f;	//Speedweight
//
//			vector2f a = new vector2f(_P2.X - _P1.X, 
//									  _P2.Y - _P1.Y);
//
//			vector2f b = new vector2f(_P3.X - _P2.X, 
//									  _P3.Y - _P2.Y);
//
//			w1 = (float)w1*a.length/Vmax;
//			w2 = 1- w1;
//
//			//Prevent Zero Values
//			double al = Math.Max(a.length,0.01);
//			double bl = Math.Max(b.length,0.01);
//
//			double alxbl = al * bl;
//
//			_Cost = (float)(w1*(1-Math.Max(a*b/alxbl,0)) + 
//				            w2*(1-2*Math.Sqrt(alxbl)/(al + bl)));
//				
//			vector2f a = new vector2f(P2.X - P1.X, 
//									  P2.Y - P1.Y);
//
//			vector2f b = new vector2f(P3.X - P2.X,
//									  P3.Y - P2.Y);
//
//			float w1 = 0.5f;
//			float w2 = 0.5f;
//
//			double dx = Math.Abs(a.X - b.X);
//			double dy = Math.Abs(a.Y - b.Y);
//
//			return (float)(w1*dx/_Vmax + w2*dy/_Vmax);

//		}

