using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Runtime.InteropServices;
using System.IO;
using System.Diagnostics;

using DShowNET;
using DShowNET.Device;

namespace MTClient
{
	/// <summary>
	/// Zusammenfassung fr clsFilterGraphController.
	/// </summary>
	public class clsFilterGraphController: ISampleGrabberCB
	{
		private DeviceSelector _selector;

		private System.Windows.Forms.Panel			_myParentPanel;
		private System.Windows.Forms.Form 			_myParent;
		/// <summary> flag to detect first Form appearance </summary>
		private bool					_firstActive;
		/// <summary> file name for AVI. </summary>
		private string					_fileName;

		/// <summary> filter Graph interface. </summary>
		private IFilterGraph            _filterGraph;

		/// <summary> Graph Builder interface. </summary>
		private IGraphBuilder			_graphBuilder;

		/// <summary> Null Renderer </summary>
		private IBaseFilter             _nullRenderer;

		/// <summary> Null Renderer </summary>
		private IBaseFilter             _nullRenderer2;

		/// <summary> Infinite Pin Tee Filter </summary>
		private IBaseFilter             _infTee;

		/// <summary> SWFilter Transform Filter </summary>
		private IBaseFilter				_transformFilter;

		IBaseFilter		 _mux = null;
		IFileSinkFilter	 _sink = null;
		
		/// <summary> grabber filter interface. </summary>
		private IBaseFilter				_baseGrabFlt;

		/// <summary> control interface. </summary>
		private IMediaControl			_mediaCtrl;

		/// <summary> graph event interface. </summary>
		private IMediaEventEx			_mediaEvt;

		/// <summary> video preview window interface. </summary>
		private IVideoWindow			_videoWin;

		/// <summary> capture graph builder interface. </summary>
		private ISampleGrabber			_sampGrabber;

		/// <summary> structure describing the bitmap to grab. </summary>
		private	VideoInfoHeader			_videoInfoHeader;
		private	bool					_captured = true;
		private	int						_bufferedSize;

		/// <summary> list of installed video devices. </summary>
		private ArrayList				_capDevices;

		/// <summary> base filter of the actually used video devices. </summary>
		private IBaseFilter				_capFilter;

		/// <summary> capture graph builder interface. </summary>
		private ICaptureGraphBuilder2	_capGraph;

		/// <summary> buffer for bitmap data. </summary>
		private	byte[]					_savedArray;

		/// <summary> event when callback has finished (ISampleGrabberCB.BufferCB). </summary>
		private delegate void CaptureDone();

		private const int WM_GRAPHNOTIFY	= 0x00008001;	// message from graph

		private const int WS_CHILD			= 0x40000000;	// attributes for video window
		private const int WS_CLIPCHILDREN	= 0x02000000;
		private const int WS_CLIPSIBLINGS	= 0x04000000;
		#if DEBUG
			private int		rotCookie = 0;
		#endif

		public clsFilterGraphController()
		{			
			//
			// TODO: Fgen Sie hier die Konstruktorlogik hinzu
			//
		}

		public void setHandle(System.Windows.Forms.Panel  ParentPanel, System.Windows.Forms.Form Parent)
		{
			_myParentPanel = ParentPanel;
			_myParent = Parent;
		}

		public void SaveCaptureFile()
		{
			SaveFileDialog sd = new SaveFileDialog();
			sd.FileName = @"CaptureNET.avi";
			sd.Title = "Save Video Stream as...";
			sd.Filter = "Video file (*.avi)|*.avi";
			sd.FilterIndex = 1;
			if( sd.ShowDialog() != DialogResult.OK )
			{ 
				return; 
			}
			_fileName = sd.FileName;
		}

		/// <summary> detect first form appearance, start capturing. </summary>
		public void VideoActivated()
		{
			if( _firstActive )
				return;
			_firstActive = true;

			if( ! DsUtils.IsCorrectDirectXVersion() )
			{
				Console.WriteLine("DirectX 8.1 NOT installed!" );
				_myParent.Close(); 
				return;
			}

			if( ! DsDev.GetDevicesOfCat( FilterCategory.VideoInputDevice, out _capDevices ) )
			{
				Console.WriteLine("No video capture devices found!");
				_myParent.Close(); 
				return;
			}

			DsDevice dev = null;
			if( _capDevices.Count == 1 )
				dev = _capDevices[0] as DsDevice;
			else
			{
				_selector = new DeviceSelector( _capDevices );
				_selector.ShowDialog();
				dev = _selector.SelectedDevice;
			}

			if( dev == null )
			{
				_myParent.Close(); 
				return;
			}

			if( ! StartupVideo( dev.Mon ) )
			{	_myParent.Close();}
		}

		/// <summary> start all the interfaces, graphs and preview window. </summary>
		public bool StartupVideo( UCOMIMoniker mon )
		{
			int hr;
			try 
			{
				if( ! CreateCaptureDevice( mon ) )
					return false;

				if( ! GetInterfaces() )
					return false;

				if( ! SetupGraph() )
					return false;

				if( ! SetupVideoWindow() )
					return false;

				#if DEBUG
					DsROT.AddGraphToRot( _graphBuilder, out rotCookie );		// graphBuilder capGraph
				#endif
				hr = _mediaCtrl.Run();
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine( "Could not start video stream\r\n" + ee.Message);
				return false;
			}
		}

		/// <summary> create the used COM components and get the interfaces. </summary>
		public bool GetInterfaces()
		{
			Type comtype = null;
			object comobj = null;
			try 
			{
				comtype = Type.GetTypeFromCLSID( Clsid.FilterGraph );
				if( comtype == null )
					throw new NotImplementedException( @"DirectShow FilterGraph not installed/registered!" );
				comobj = Activator.CreateInstance( comtype );
				_graphBuilder = (IGraphBuilder) comobj; comobj = null;

				comtype = Type.GetTypeFromCLSID( Clsid.SampleGrabber );
				if( comtype == null )
					throw new NotImplementedException( @"DirectShow SampleGrabber not installed/registered!" );
				comobj = Activator.CreateInstance( comtype );
				Console.WriteLine("SampleGrabber wurde erzeugt");
				_sampGrabber = (ISampleGrabber) comobj; comobj = null;

				comtype = Type.GetTypeFromCLSID( Clsid.InfTee);
				if( comtype == null )
					throw new NotSupportedException( "DirectX (8.1 or higher) not installed?" );
				comobj = Activator.CreateInstance(comtype);
				Console.WriteLine("Infinite Pin Tee Filter wurde erzeugt");
				_infTee = (IBaseFilter) comobj; comobj = null;

				comtype = Type.GetTypeFromCLSID( Clsid.NullRenderer);
				if( comtype == null )
					throw new NotSupportedException( "DirectX (8.1 or higher) not installed?" );
				comobj = Activator.CreateInstance(comtype);
				Console.WriteLine("Null Renderer wurde erzeugt");
				_nullRenderer = (IBaseFilter) comobj; comobj = null;

				comtype = Type.GetTypeFromCLSID( Clsid.NullRenderer);
				if( comtype == null )
					throw new NotSupportedException( "DirectX (8.1 or higher) not installed?" );
				comobj = Activator.CreateInstance(comtype);
				Console.WriteLine("Null Renderer wurde erzeugt");
				_nullRenderer2 = (IBaseFilter) comobj; comobj = null;

				comtype = Type.GetTypeFromCLSID( Clsid.SWFilter );
				if( comtype == null )
					throw new NotSupportedException( "DirectX (8.1 or higher) not installed?" );
				comobj = Activator.CreateInstance(comtype);
				Console.WriteLine("Transform Filter wurde erzeugt");
				_transformFilter = (IBaseFilter) comobj; comobj = null;

				Guid clsid = Clsid.CaptureGraphBuilder2;
				Guid riid = typeof(ICaptureGraphBuilder2).GUID;
				comobj = DsBugWO.CreateDsInstance( ref clsid, ref riid );
				Console.WriteLine("Capture Graph wurde erzeugt");
				_capGraph = (ICaptureGraphBuilder2) comobj; comobj = null;

				_mediaCtrl	= (IMediaControl)	_graphBuilder;
				_videoWin	= (IVideoWindow)	_graphBuilder;
				_mediaEvt	= (IMediaEventEx)	_graphBuilder;
				_baseGrabFlt	= (IBaseFilter)	_sampGrabber;
				_filterGraph = (IFilterGraph)	_graphBuilder;
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine("Could not get interfaces\r\n" + ee.Message );
				return false;
			}
			finally
			{
				if( comobj != null )
					Marshal.ReleaseComObject( comobj ); comobj = null;
			}
		}

		/// <summary> build the capture graph. </summary>
		public bool SetupGraph()
		{
			int hr;
			IPin					pinOut;
			IPin					pinIn;
			IPin					pinOutRender;

			try 
			{
				//*************************************************
				// Grab Init
				//*************************************************
				AMMediaType media = new AMMediaType();
				media.majorType	= MediaType.Video;
				media.subType	= MediaSubType.RGB24;
				media.formatType = FormatType.VideoInfo;		// ???
				hr = _sampGrabber.SetMediaType( media );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END Grab Init
				//*************************************************


				hr = _capGraph.SetFiltergraph( _graphBuilder );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
			
				hr = _graphBuilder.AddFilter( _capFilter, "Ds.NET Video Capture Device" );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				//DsUtils.ShowPinDialog(_capFilter ,_myParentPanel.Handle );

				hr= DsUtils.GetPin(_capFilter,PinDirection.Output  ,0, out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr = _graphBuilder.AddFilter( _infTee, "Ds.NET Infinite Pin Tee Filter" );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
			
				hr= DsUtils.GetPin(_infTee,PinDirection.Input ,0, out pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr = _graphBuilder.Connect(pinOut, pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				//***********************************************************
				//  SampleGrabber Filter
				//***********************************************************
				hr= DsUtils.GetPin(_infTee,PinDirection.Output,0, out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);
			
				hr = _graphBuilder.AddFilter(_nullRenderer,"Null Renderer");
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				//Add SampleGrabber Filter 
				hr = _graphBuilder.AddFilter(_baseGrabFlt, "Ds.NET Grabber");
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );			
				//Get Input Pin from SampleGrabber Filter
				hr= DsUtils.GetPin(_baseGrabFlt, PinDirection.Input,0, out pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);	
		
				hr = _graphBuilder.Connect(pinOut, pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );	
			
				hr= DsUtils.GetPin(_baseGrabFlt, PinDirection.Output,0, out pinOutRender);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);
			
				hr= DsUtils.GetPin(_nullRenderer, PinDirection.Input ,0,out pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr = _graphBuilder.Connect(pinOutRender, pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END SampleGrabber Filter
				//*************************************************

				//*************************************************
				// Preview Filter
				//*************************************************
				hr= DsUtils.GetPin(_infTee,PinDirection.Output,1, out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);
			
				hr =_graphBuilder.Render(pinOut);
				if( hr < 0)
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END Preview Filter
				//*************************************************

				//***********************************************************
				//  Transform Filter
				//***********************************************************
				hr= DsUtils.GetPin(_infTee,PinDirection.Output,2, out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr = _graphBuilder.AddFilter(_nullRenderer2,"Null Renderer");
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				//Add Transform Filter
				hr = _graphBuilder.AddFilter(_transformFilter,"SWFilter");
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
					
				//Property Window Transform Filter
				//DsUtils.ShowPinDialog(transformFilter,myParent.Handle );

				//Get Input Pin from Transform Filter
				hr= DsUtils.GetPin(_transformFilter, PinDirection.Input,0, out pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);		
							
				hr = _graphBuilder.Connect(pinOut, pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				hr= DsUtils.GetPin(_transformFilter, PinDirection.Output,0,out pinOut);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr= DsUtils.GetPin(_nullRenderer2, PinDirection.Input ,0,out pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR(hr);

				hr = _graphBuilder.Connect(pinOut, pinIn);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END Transform Filter
				//*************************************************
				Console.WriteLine("Aktueller Aufbau der Filter:");
				hr = DsUtils.EnumFilters(_filterGraph);
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				//*************************************************
				//SampleGrabber Final
				//*************************************************
				media = new AMMediaType();
				hr = _sampGrabber.GetConnectedMediaType( media );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				if( (media.formatType != FormatType.VideoInfo) || (media.formatPtr == IntPtr.Zero) )
					throw new NotSupportedException( "Unknown Grabber Media Format" );

				_videoInfoHeader = (VideoInfoHeader) Marshal.PtrToStructure( media.formatPtr, typeof(VideoInfoHeader) );
				Marshal.FreeCoTaskMem( media.formatPtr ); media.formatPtr = IntPtr.Zero;

				hr = _sampGrabber.SetBufferSamples( false );
				if( hr == 0 )
					hr = _sampGrabber.SetOneShot( false );
				if( hr == 0 )
					hr = _sampGrabber.SetCallback( null, 0 );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				//*************************************************
				// END SampleGrabber Final 
				//*************************************************
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine("Could not setup graph\r\n" + ee.Message );
				return false;
			}
			finally
			{

			}
		}

		/// <summary> create the user selected capture device. </summary>
		public bool CreateCaptureDevice( UCOMIMoniker mon )
		{
			object capObj = null;
			try 
			{
				Guid gbf = typeof( IBaseFilter ).GUID;
				mon.BindToObject( null, null, ref gbf, out capObj );
				_capFilter = (IBaseFilter) capObj; capObj = null;
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine( "Could not create capture device\r\n" + ee.Message);
				return false;
			}
			finally
			{
				if( capObj != null )
					Marshal.ReleaseComObject( capObj ); capObj = null;
			}

		}


		/// <summary> make the video preview window to show in videoPanel. </summary>
		public bool SetupVideoWindow()
		{
			int hr;
			try 
			{
				// Set the video window to be a child of the main window
				hr = _videoWin.put_Owner( _myParentPanel.Handle );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				// Set video window style
				hr = _videoWin.put_WindowStyle( WS_CHILD | WS_CLIPCHILDREN );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				// Use helper function to position video window in client rect of owner window
				ResizeVideoWindow();

				// Make the video window visible, now that it is properly positioned
				hr = _videoWin.put_Visible( DsHlp.OATRUE );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );

				hr = _mediaEvt.SetNotifyWindow( _myParentPanel.Handle, WM_GRAPHNOTIFY, IntPtr.Zero );
				if( hr < 0 )
					Marshal.ThrowExceptionForHR( hr );
				return true;
			}
			catch( Exception ee )
			{
				Console.WriteLine( "Could not setup video window\r\n" + ee.Message );
				return false;
			}
		}
		public void PropertyCap()
		{
			//Property Capture
			DsUtils.ShowCapPinDialog( _capGraph, _capFilter, _myParentPanel.Handle );	
		}

		public void PropertyPrev()
		{
			//Property Webcam
			DsUtils.ShowPrevPinDialog( _capGraph, _capFilter, _myParentPanel.Handle );	
		}

		public void PropertyTrans()
		{	
			//Property Window Transform Filter
			DsUtils.ShowPinDialog(_transformFilter,_myParent.Handle );
		}


		public IAMVideoProcAmp IAMVideoProcAmpInit()
		{
			
			int hr;
			object comObj = null;
			ICaptureGraphBuilder2 bld = _capGraph;
			IBaseFilter flt = _capFilter;


			try 
			{
				Guid cat = PinCategory.Preview;
				Guid type = MediaType.Video;
				Guid iid = typeof(IAMVideoProcAmp).GUID;
				hr = bld.FindInterface(ref cat, ref type, flt, ref iid, out comObj);
				if( hr != 0 )
				{
					return null;
				}

				IAMVideoProcAmp videoProc=null;
				videoProc=comObj as IAMVideoProcAmp;
				if(videoProc == null) 
				{
					Console.WriteLine("IAMVideoProcAmp not present");
					return null;
				}
				
				return videoProc;

			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: ShowPrevPinDialog " + ee.Message );
				return null;
			}
			finally
			{
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}

		public IAMVideoControl IAMVideoControlInit()
		{
			
			int hr;
			object comObj = null;
			ICaptureGraphBuilder2 bld = _capGraph;
			IBaseFilter flt = _capFilter;


			try 
			{
				Guid cat = PinCategory.Preview;
				Guid type = MediaType.Video;
				Guid iid = typeof(IAMVideoControl).GUID;
				hr = bld.FindInterface(ref cat, ref type, flt, ref iid, out comObj);
				if( hr != 0 )
				{
					return null;
				}

				IAMVideoControl videoControl=null;
				videoControl=comObj as IAMVideoControl;
				if(videoControl == null) 
				{
					Console.WriteLine("IAMVideoControl not present");
					return null;
				}
				
				return videoControl;

			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: ShowPrevPinDialog " + ee.Message );
				return null;
			}
			finally
			{
				if( comObj != null )
					Marshal.ReleaseComObject( comObj ); comObj = null;
			}
		}

		public IAMStreamConfig IAMStreamConfigInit()
		{
			int hr;
			object comObj = null;
			ICaptureGraphBuilder2 bld = _capGraph;
			IBaseFilter flt = _capFilter;

			try 
			{

				Guid cat  = PinCategory.Capture;
				Guid type = MediaType.Interleaved;
				Guid iid = typeof(IAMStreamConfig).GUID;

				hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
				if( hr != 0 )
				{
					type = MediaType.Video;
					hr = bld.FindInterface( ref cat, ref type, flt, ref iid, out comObj );
					
					if( hr != 0 )
						return null;
				}
				
		
				//Instead of calling OleCreatePropertyFrame do ...
				IAMStreamConfig streamConfig=null;
				streamConfig=comObj as IAMStreamConfig;

				if(streamConfig == null) 
				{
					Console.WriteLine("IAMStreamConfig not present");
					return null;
				}

				return streamConfig;

			}
			catch( Exception ee )
			{
				Console.WriteLine( "!Ds.NET: IAMStreamConfig " + ee.Message );
				return null;
			}
			finally
			{
				if( comObj != null )
					Marshal.ReleaseComObject( comObj); comObj = null;
			}
		}

		/// <summary> if the videoPanel is resized, adjust video preview window. </summary>
		public void ResizeVideoWindow()
		{
			if( _videoWin != null )
			{
				Rectangle rc = _myParentPanel.ClientRectangle;
				_videoWin.SetWindowPosition( 0, 0, rc.Right, rc.Bottom );
			}
		}


		/// <summary> graph event (WM_GRAPHNOTIFY) handler. </summary>
		public void OnGraphNotify()
		{
			if( _mediaEvt != null )
			{
				DsEvCode	code;
				int p1, p2, hr = 0;
				do
				{
					hr = _mediaEvt.GetEvent( out code, out p1, out p2, 0 );
					if( hr < 0 )
						break;
					hr = _mediaEvt.FreeEventParams( code, p1, p2 );
				}
				while( hr == 0 );
			}
			
		}


		/// <summary> do cleanup and release DirectShow. </summary>
		public void CloseInterfaces()
		{
			int hr;
			try 
			{
				#if DEBUG
					if( rotCookie != 0 )
						DsROT.RemoveGraphFromRot( ref rotCookie );
				#endif
				if( _mediaCtrl != null )
				{
					hr = _mediaCtrl.Stop();
					_mediaCtrl = null;
				}

				if( _mediaEvt != null )
				{
					hr = _mediaEvt.SetNotifyWindow( IntPtr.Zero, WM_GRAPHNOTIFY, IntPtr.Zero );
					_mediaEvt = null;
				}

				if( _videoWin != null )
				{
					hr = _videoWin.put_Visible( DsHlp.OAFALSE );
					hr = _videoWin.put_Owner( IntPtr.Zero );
					_videoWin = null;
				}

				if( _capGraph != null )
					Marshal.ReleaseComObject( _capGraph ); _capGraph = null;

				if( _graphBuilder != null )
					Marshal.ReleaseComObject( _graphBuilder ); _graphBuilder = null;

				if( _capFilter != null )
					Marshal.ReleaseComObject( _capFilter ); _capFilter = null;
			
				if( _mux != null )
					Marshal.ReleaseComObject( _mux ); _mux = null;
				if( _sink != null )
					Marshal.ReleaseComObject( _sink ); _sink = null;

				if( _capDevices != null )
				{
					foreach( DsDevice d in _capDevices )
						d.Dispose();
					_capDevices = null;
				}

				_baseGrabFlt = null;
				if( _sampGrabber != null )
					Marshal.ReleaseComObject( _sampGrabber ); _sampGrabber = null;
			}
			catch( Exception )
			{}
		}

		public void PreviewOn()
		{
			IPin pinOut;
			
			int hr = _mediaCtrl.Stop();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			hr= DsUtils.GetPin(_infTee,PinDirection.Output,1, out pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			hr = _graphBuilder.Render(pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			Console.WriteLine("Eingeschaltet: Preview wurde eingeschaltet");	
			hr = DsUtils.EnumFilters(_filterGraph);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			if( ! SetupVideoWindow() )
				return;

			hr = _mediaCtrl.Run();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			
		}
		public void PreviewOff()
		{
			IPin pinOut;
			IBaseFilter remFilter;
			
			int hr = _mediaCtrl.Stop();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			hr= DsUtils.GetPin(_infTee,PinDirection.Output,1, out pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			hr = _graphBuilder.Disconnect(pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			hr = _graphBuilder.FindFilterByName("Video Renderer", out remFilter);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			hr = _graphBuilder.RemoveFilter(remFilter);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			hr = _graphBuilder.FindFilterByName("Color Space Converter", out remFilter);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			hr = _graphBuilder.RemoveFilter(remFilter);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			Console.WriteLine("Ausgeschaltet: Preview wurde ausgeschaltet");	
			hr = DsUtils.EnumFilters(_filterGraph);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			_myParentPanel.Refresh();

			hr = _mediaCtrl.Run();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
		}

		public void CaptureOn()
		{
			IPin pinOut;
			IPin pinIn;

			int hr = _mediaCtrl.Stop();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			//***********************
			// MUX
			//***********************
			hr= DsUtils.GetPin(_infTee,PinDirection.Output,3, out pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);			
			
			Guid sub = MediaSubType.Avi;
			hr = _capGraph.SetOutputFileName( ref sub, _fileName, out _mux, out _sink );
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );		

			hr= DsUtils.GetPin(_mux,PinDirection.Input,0, out pinIn);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			hr = _graphBuilder.Connect(pinOut,pinIn);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			Console.WriteLine("Eingeschaltet: AVI Mux und File Writer:");
			hr = DsUtils.EnumFilters(_filterGraph);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			//***********************
			// END MUX
			//***********************

			hr = _mediaCtrl.Run();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
		}

		public void Stop()
		{
			int hr = _mediaCtrl.Stop();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);
		}

		public void Start()
		{
			int hr = _mediaCtrl.Run();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);
		}
		public void CaptureOff()
		{
			IPin pinOut;
			IBaseFilter remsink =null ;

			int hr = _mediaCtrl.Stop();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);
			
			hr= DsUtils.GetPin(_infTee,PinDirection.Output,3, out pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			hr = _graphBuilder.Disconnect(pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
					
			hr = _graphBuilder.RemoveFilter(_mux);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
					
			remsink = _sink as IBaseFilter;

			hr = _graphBuilder.RemoveFilter(remsink);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );

			Console.WriteLine("Ausgeschaltet: AVI Mux und File Writer:");	
			hr = DsUtils.EnumFilters(_filterGraph);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
			
			hr = _mediaCtrl.Run();
			if( hr < 0 )
				Marshal.ThrowExceptionForHR( hr );
		}

		

		public void  GrabImage()
		{
			Console.WriteLine( "!!BTN: toolBarBtnGrab" );

			if( _savedArray == null )
			{
				int size = _videoInfoHeader.BmiHeader.ImageSize;
				if( (size < 1000) || (size > 16000000) )
					return;
				_savedArray = new byte[ size + 64000 ];
			}


//			Image old = pictureBox.Image;
//			pictureBox.Image = null;
//			if( old != null )
//				old.Dispose();

			_captured = false;
			int hr = _sampGrabber.SetCallback(this, 1 );
		}

		int ISampleGrabberCB.SampleCB( double SampleTime, IMediaSample pSample )
		{
			Console.WriteLine( "!!CB: ISampleGrabberCB.SampleCB" );
			return 0;
		}

		/// <summary> buffer callback, COULD BE FROM FOREIGN THREAD. </summary>
		int ISampleGrabberCB.BufferCB( double SampleTime, IntPtr pBuffer, int BufferLen )
		{
			if( (_captured || _savedArray == null) )
			{
				Console.WriteLine( "!!CB: ISampleGrabberCB.BufferCB" );
				return 0;
			}

			_captured = true;
			_bufferedSize = BufferLen;
			Console.WriteLine( "!!CB: ISampleGrabberCB.BufferCB  !GRAB! size = " + BufferLen.ToString() );
			if( (pBuffer != IntPtr.Zero) && (BufferLen > 1000) && (BufferLen <= _savedArray.Length) )
				Marshal.Copy( pBuffer, _savedArray, 0, BufferLen );
			else
				Console.WriteLine( "    !!!GRAB! failed " );
			_myParentPanel.BeginInvoke( new CaptureDone( OnCaptureDone ) );
			return 0;
		}

		public void OnCaptureDone()
		{
			Console.WriteLine( "!!DLG: OnCaptureDone" );
			try 
			{
				//toolBarBtnGrab.Enabled = true;
				int hr;
				if( _sampGrabber == null )
					return;
				hr = _sampGrabber.SetCallback( null, 0 );

				int w = _videoInfoHeader.BmiHeader.Width;
				int h = _videoInfoHeader.BmiHeader.Height;
				if( ((w & 0x03) != 0) || (w < 32) || (w > 4096) || (h < 32) || (h > 4096) )
					return;
				int stride = w * 3;

				GCHandle handle = GCHandle.Alloc( _savedArray, GCHandleType.Pinned );
				int scan0 = (int) handle.AddrOfPinnedObject();
				scan0 += (h - 1) * stride;
				Bitmap b = new Bitmap( w, h, -stride, PixelFormat.Format24bppRgb, (IntPtr) scan0 );
				handle.Free();
				_savedArray = null;
//				Image old = pictureBox.Image;
//				pictureBox.Image = b;
//				if( old != null )
//					old.Dispose();


			}
			catch( Exception ee )
			{
				Console.WriteLine("Could not grab picture\r\n" + ee.Message, "DirectShow.NET");
			}
		}

		public void VideoControl(IAMVideoControl videoControl)
		{
			IPin pinOut;
			Size test = new Size(160,120);
			int listsize;
			long[] framerates = new long[6];

			int hr= DsUtils.GetPin(_capFilter,PinDirection.Output,0, out pinOut);
			if( hr < 0 )
				Marshal.ThrowExceptionForHR(hr);

			VideoControlFlags flagcontrol;
			videoControl.GetCaps(pinOut,out flagcontrol);
			videoControl.GetMode(pinOut,out flagcontrol);
			Console.WriteLine(flagcontrol.ToString());
			long frame;
			//videoControl.GetCurrentActualFrameRate(pinOut,out frame);
			videoControl.GetMaxAvailableFrameRate(pinOut,0, test, out frame);
			videoControl.GetFrameRateList(pinOut,0,test,out listsize, framerates);
			videoControl.SetMode(pinOut, VideoControlFlags.ExternalTriggerEnable);

			videoControl.GetMode(pinOut,out flagcontrol);
			Console.WriteLine(flagcontrol.ToString());
			Console.WriteLine("Max Available Framerate: " + frame.ToString());
			for(int i=0; i<listsize;i++)
			{
				Console.WriteLine("Framerate " + i + ": " + framerates[i].ToString());
			}
		}
	}
}
