using System;
using MT.Network;
using tsai;
using MT.Tracker.DataStructures;

using System.Collections;

namespace MTServer
{
	/// <summary>
	/// Zusammenfassung fr clsAssociation.
	/// </summary>
	public class clsAssociation
	{
		int			_framenr			= 1;
		float		_size				= 165;
		double[,]	_3Dmodel			= new double[14,3];	// 3D-Modell
		double[,]	_2Dmodel			= new double[14,2];	// 2D-Modell
		float[,]	_Featurepoint		= new float[12,2];	// Zuordnung FP-BP

		const int	_BODYPOINTCOUNT		= 13;
		private TUKMain	_TUK_Tracker;
		private tsai_converter[] _calib;
		


		private vector3f[] _cameracoords;

		private FeaturePoint[][] _FPs;

		private clsClientSynchronizer _ClientSynchronizer;

		private frm3DView _GL3DView;
		
		clsClientCollection	_ClientCollection;
		tsai_converter		_tsai;

		FeaturePointFrame	_lastFrame;
		


		#region "Properties"
		public float Size
		{
			get {return _size; }
			set {_size = value;}
		}

		public float[,] FP
		{
			get {return _Featurepoint; }
		}

		public FeaturePoint[][] FPs
		{
			get {return _FPs; }
		}

		public double[,] Model3D
		{
			get {return _3Dmodel; }
		}

		public double[,] Model2D
		{
			get {return _2Dmodel; }
		}

		public FeaturePointFrame lastFrame
		{
			get {return _lastFrame; }
		}

		public clsClientCollection ClientCollection
		{
			get {return _ClientCollection; }
		}
		#endregion

		public void Initialize(clsAssociation Association, clsClientSynchronizer ClientSynchronizer)
		{
			_ClientSynchronizer = ClientSynchronizer;
		}
		

		public clsAssociation(clsClientCollection ClientCollection)
		{
			_ClientCollection = ClientCollection;
			
			//Hals (kein Bodypoint, nur zum zeichnen bentigt)
			_3Dmodel[13,0] = 0;
			_3Dmodel[13,1] = 0;
			_3Dmodel[13,2] = 1600;

			bool modellausgestreckt = true;

			if(modellausgestreckt)
			{
				//Kopf (grn)
				_3Dmodel[8,0] = 0;
				_3Dmodel[8,1] = 0;
				_3Dmodel[8,2] = 1800;
			
				//linke Hand (rot)
				_3Dmodel[0,0] = 0;
				_3Dmodel[0,1] = -800;
				_3Dmodel[0,2] = 1600;
			
				//linker Ellbogen (blau)
				_3Dmodel[4,0] = 0;
				_3Dmodel[4,1] = -500;
				_3Dmodel[4,2] = 1600;
			
				//linke Schulter (grn)
				_3Dmodel[9,0] = 0;
				_3Dmodel[9,1] = -200;
				_3Dmodel[9,2] = 1600;
			
				//rechte Schulter (rot)
				_3Dmodel[1,0] = 0;
				_3Dmodel[1,1] = 200;
				_3Dmodel[1,2] = 1600;
			
				//rechter Ellbogen (grn)
				_3Dmodel[10,0] = 0;
				_3Dmodel[10,1] = 500;
				_3Dmodel[10,2] = 1600;
			
				//rechte Hand (blau)
				_3Dmodel[5,0] = 0;
				_3Dmodel[5,1] = 800;
				_3Dmodel[5,2] = 1600;
			
				//linke Hfte (grn)
				_3Dmodel[11,0] = 0;
				_3Dmodel[11,1] = -200;
				_3Dmodel[11,2] = 1100;
			
				//rechte Hfte (rot)
				_3Dmodel[2,0] = 0;
				_3Dmodel[2,1] = 200;
				_3Dmodel[2,2] = 1100;
			
				//linkes Knie (blau)
				_3Dmodel[6,0] = 0;
				_3Dmodel[6,1] = -200;
				_3Dmodel[6,2] = 600;
			
				//rechtes Knie (grn)
				_3Dmodel[12,0] = 0;
				_3Dmodel[12,1] = 200;
				_3Dmodel[12,2] = 600;
			
				//linker Fuss (rot)
				_3Dmodel[3,0] = 0;
				_3Dmodel[3,1] = -200;
				_3Dmodel[3,2] = 200;
			
				//rechter Fuss (blau)
				_3Dmodel[7,0] = 0;
				_3Dmodel[7,1] = 200;
				_3Dmodel[7,2] = 200;
			}
			else
			{
				//Kopf (grn)
				_3Dmodel[8,0] = 0;
				_3Dmodel[8,1] = 0;
				_3Dmodel[8,2] = 1800;
			
				//linke Hand (rot)
				_3Dmodel[0,0] = 0;
				_3Dmodel[0,1] = -200;
				_3Dmodel[0,2] = 1200;
			
				//linker Ellbogen (blau)
				_3Dmodel[4,0] = 0;
				_3Dmodel[4,1] = -400;
				_3Dmodel[4,2] = 1200;
			
				//linke Schulter (grn)
				_3Dmodel[9,0] = 0;
				_3Dmodel[9,1] = -200;
				_3Dmodel[9,2] = 1600;
			
				//rechte Schulter (rot)
				_3Dmodel[1,0] = 0;
				_3Dmodel[1,1] = 200;
				_3Dmodel[1,2] = 1600;
			
				//rechter Ellbogen (grn)
				_3Dmodel[10,0] = 0;
				_3Dmodel[10,1] = 400;
				_3Dmodel[10,2] = 1450;
			
				//rechte Hand (blau)
				_3Dmodel[5,0] = 0;
				_3Dmodel[5,1] = 200;
				_3Dmodel[5,2] = 1200;
			
				//linke Hfte (grn)
				_3Dmodel[11,0] = 0;
				_3Dmodel[11,1] = -200;
				_3Dmodel[11,2] = 1100;
			
				//rechte Hfte (rot)
				_3Dmodel[2,0] = 0;
				_3Dmodel[2,1] = 200;
				_3Dmodel[2,2] = 1100;
			
				//linkes Knie (blau)
				_3Dmodel[6,0] = 0;
				_3Dmodel[6,1] = -200;
				_3Dmodel[6,2] = 600;
			
				//rechtes Knie (grn)
				_3Dmodel[12,0] = 0;
				_3Dmodel[12,1] = 200;
				_3Dmodel[12,2] = 600;
			
				//linker Fuss (rot)
				_3Dmodel[3,0] = 0;
				_3Dmodel[3,1] = -200;
				_3Dmodel[3,2] = 200;
			
				//rechter Fuss (blau)
				_3Dmodel[7,0] = 0;
				_3Dmodel[7,1] = 200;
				_3Dmodel[7,2] = 200;
			
			}

			// Offset
			for (int i=0; i<_3Dmodel.GetLength(0); i++)
			{
				_3Dmodel[i,0] += 600;
				_3Dmodel[i,1] += 600;
				_3Dmodel[i,2] += 0;
			}
		}


		public void Associate()
		{
			double xc = new double();
			double yc = new double();
			double zc = new double();
			double x = new double();
			double y = new double();
			double min;
			double tmp;
			int i, k;
			int m=0;

			_FPs = new FeaturePoint[_BODYPOINTCOUNT][];
				
			for(int n=0; n<_BODYPOINTCOUNT; n++)
			{
				_FPs[n] = new FeaturePoint[_ClientCollection.Count];
			}
			_cameracoords = new vector3f[_ClientCollection.Count];
			_calib		  = new tsai_converter[_ClientCollection.Count];

			
			

			foreach (clsClient Client in _ClientCollection.Values)
			{

				if (Client.FPData.FeaturePointFrames.Count!=0 && Client.Tsai!=null)
				{
					_lastFrame	= (FeaturePointFrame)Client.FPData.FeaturePointFrames.GetByIndex(Client.FPData.FeaturePointFrames.Count-1);
					_tsai		= (tsai_converter)Client.Tsai;
					

					
					_tsai.get_camera_position(ref xc, ref yc, ref zc);
					_cameracoords[m] = new vector3f((float)xc, (float)yc, (float)zc);
					_calib[m]		 = _tsai;
					
					Resize3Dmodel();
			
					foreach (FeaturePoint p in _lastFrame.Pred.Values)
					{
						min = 999999999; tmp = 0; i = 0;	
						for (k=0; k<=3; k++)
						{				
							tmp = Math.Abs(p.X-_2Dmodel[k,0]) + Math.Abs(p.Y-_2Dmodel[k,1]);
							if( tmp < min)
							{
								min = tmp;
								i=k;
							}
							
						}
						_FPs[i][m] = p;
					}

					foreach (FeaturePoint p in _lastFrame.Pblue.Values)
					{
						min = 999999999; tmp = 0; i = 0;	
						for (k=4; k<=7; k++)
						{				
							tmp = Math.Abs(p.X-_2Dmodel[k,0]) + Math.Abs(p.Y-_2Dmodel[k,1]);
							if( tmp < min)
							{
								min = tmp;
								i=k;
							}
							
						}
						_FPs[i][m] = p;
					}

					foreach (FeaturePoint p in _lastFrame.Pgreen.Values)
					{
						min = 999999999; tmp = 0; i = 0;	
						for (k=8; k<=12; k++)
						{				
							tmp = Math.Abs(p.X-_2Dmodel[k,0]) + Math.Abs(p.Y-_2Dmodel[k,1]);
							if( tmp < min)
							{
								min = tmp;
								i=k;
							}
							
						}
						_FPs[i][m] = p;
					}
				}
				m++;
			}
			_GL3DView = new frm3DView();
			setTracker();
			start3DView();
		}


		private void start3DView()
		{
			
			_GL3DView.Initialize(_TUK_Tracker);
			_GL3DView.Show();
		}


		public void setTracker()
		{
			_framenr=1;
	
			//_TUK_Tracker = new TUKMain( _calib, _cameracoords, _ClientCollection);
			_ClientSynchronizer.InitializeTUKMain(_GL3DView);
			_TUK_Tracker = _ClientSynchronizer.tukMain;
			//_TUK_Tracker.Reset();
			_TUK_Tracker.CreateBodyPoint("Hand_Links"		, eColorCode.R, _FPs[0]);
			_TUK_Tracker.CreateBodyPoint("Hand_Rechts"		, eColorCode.B, _FPs[5]);
			_TUK_Tracker.CreateBodyPoint("Huefte_Rechts"	, eColorCode.R, _FPs[2]);
			_TUK_Tracker.CreateBodyPoint("Fuss_Links"		, eColorCode.R, _FPs[3]);
			_TUK_Tracker.CreateBodyPoint("Ellbogen_Links"	, eColorCode.B, _FPs[4]);
			_TUK_Tracker.CreateBodyPoint("Schulter_Rechts"	, eColorCode.R, _FPs[1]);
			_TUK_Tracker.CreateBodyPoint("Knie_Links"		, eColorCode.B, _FPs[6]);
			_TUK_Tracker.CreateBodyPoint("Fuss_Rechts"		, eColorCode.B, _FPs[7]);
			_TUK_Tracker.CreateBodyPoint("Kopf"				, eColorCode.G, _FPs[8]);
			_TUK_Tracker.CreateBodyPoint("Schulter_Links"	, eColorCode.G, _FPs[9]);			
			_TUK_Tracker.CreateBodyPoint("Ellbogen_Rechts"	, eColorCode.G, _FPs[10]);			
			_TUK_Tracker.CreateBodyPoint("Huefte_Links"		, eColorCode.G, _FPs[11]);
			_TUK_Tracker.CreateBodyPoint("Knie_Rechts"		, eColorCode.G, _FPs[12]);

//			_TUK_Tracker.CreateBodyConnector("Stab","Fuss_Rechts","Huefte_Rechts",0.15f);

			_TUK_Tracker.CreateBodyConnector("Elle_Links","Hand_Links","Ellbogen_Links",0.15f);
			_TUK_Tracker.CreateBodyConnector("Elle_Rechts","Hand_Rechts","Ellbogen_Rechts",0.15f);
			_TUK_Tracker.CreateBodyConnector("Oberarm_Links","Ellbogen_Links","Schulter_Links",0.15f);
			_TUK_Tracker.CreateBodyConnector("Oberarm_Rechts","Ellbogen_Rechts","Schulter_Rechts",0.15f);
			_TUK_Tracker.CreateBodyConnector("Schultern","Schulter_Rechts","Schulter_Links",0.15f);
			_TUK_Tracker.CreateBodyConnector("Hals","Kopf","Schulter_Rechts","Schulter_Links",0.3f);
			_TUK_Tracker.CreateBodyConnector("Rumpf","Schulter_Rechts", "Schulter_Links", "Huefte_Rechts", "Huefte_Links",0.5f);
			_TUK_Tracker.CreateBodyConnector("Becken","Huefte_Rechts","Huefte_Links",0.15f);
			_TUK_Tracker.CreateBodyConnector("Oberschenkel_Rechts","Huefte_Rechts","Knie_Rechts",0.15f);
			_TUK_Tracker.CreateBodyConnector("Oberschenkel_Links","Huefte_Links","Knie_Links",0.15f);
			_TUK_Tracker.CreateBodyConnector("Unterschenkel_Rechts","Knie_Rechts","Fuss_Rechts",0.15f);
			_TUK_Tracker.CreateBodyConnector("Unterschenkel_Links","Knie_Links","Fuss_Links",0.15f);

			_ClientSynchronizer.StartSynchronizer();

		}

		public void nextFrame()
		{
//			int i=0;
//			FeaturePointFrame[] lastFrame = new FeaturePointFrame[_ClientCollection.Count];
//		
//			foreach (clsClient Client in _ClientCollection.Values)
//			{
//				lastFrame[i] = (FeaturePointFrame)Client.FPData.FeaturePointFrames.GetByIndex(Client.FPData.FeaturePointFrames.Count-1);
//				lastFrame[i].fid = _framenr;
//				i++;
//			}
			System.Threading.Thread Worker = new System.Threading.Thread(new System.Threading.ThreadStart(Track));
			Worker.Name = "Worker";
			Worker.IsBackground = true;
			Worker.Start();
		}

		private void Track()
		{
			while(true)
			{

				clsFPFrameArray frameArray = new clsFPFrameArray(_framenr++);
				try
				{
					foreach (clsClient Client in _ClientCollection.Values)
					{
						SortedList FPFrames = (SortedList)Client.FPData.FeaturePointFrames.SyncRoot;
						FeaturePointFrame FPFrame = (FeaturePointFrame)FPFrames.GetByIndex(Client.FPData.FeaturePointFrames.Count-1);

						frameArray.Add(Client.Host, FPFrame);
					}
				}
				catch(Exception ex)
				{
					System.Windows.Forms.MessageBox.Show(ex.Message);
				}
			
				_TUK_Tracker.TrackFrame(frameArray);

				System.Threading.Thread.Sleep(30);
			}
		}

		public void Resize3Dmodel()
		{
			double x = new double();
			double y = new double();
			
			float factor = _size/180.0f;
			
			if (_tsai!=null)
			{
				for (int i=0; i<_3Dmodel.GetLength(0); i++)
				{
					_tsai.world_coord_to_image_coord(_3Dmodel[i,0]*factor, _3Dmodel[i,1]*factor, _3Dmodel[i,2]*factor, ref x, ref y);
					_2Dmodel[i,0] = x; 			// Xi
					_2Dmodel[i,1] = y;			// Yi
				}
			}
			else
			{
				for (int i=0; i<_3Dmodel.GetLength(0); i++)
				{
					_2Dmodel[i,0] = 0; 	
					_2Dmodel[i,1] = 0;
				}
			}

		}


		public void Resize3Dmodel(string selectedClient)
		{
			double x = new double();
			double y = new double();

			
			foreach (clsClient Client in _ClientCollection.Values)
			{
				if ( Client.Host.Equals(selectedClient) )
				{
					if (Client.Tsai!=null)
					{
						_tsai = Client.Tsai;
						float factor = _size/180.0f;
						for (int i=0; i<_3Dmodel.GetLength(0); i++)
						{
							_tsai.world_coord_to_image_coord(_3Dmodel[i,0]*factor, _3Dmodel[i,1]*factor, _3Dmodel[i,2]*factor, ref x, ref y);
							_2Dmodel[i,0] = x; 			// Xi
							_2Dmodel[i,1] = y;			// Yi
						}
					}
					else
					{
						for (int i=0; i<_3Dmodel.GetLength(0); i++)
						{
							_2Dmodel[i,0] = 0; 	
							_2Dmodel[i,1] = 0;
						}
					}


					if (Client.FPData.FeaturePointFrames.Count!=0)
					{
						_lastFrame	= (FeaturePointFrame)Client.FPData.FeaturePointFrames.GetByIndex(Client.FPData.FeaturePointFrames.Count-1);
					}
				}
			}							
		}

	}
}
