using System;
using MT.Network;
using System.Collections;
using tsai;
using MT.Tracker.DataStructures;
using MT.Synchronization;

namespace MTServer
{

	/// <summary>
	/// Zusammenfassung fr clsClientCollection.
	/// </summary>
	public class clsClientCollection: SynchronizerSortedList
	{
		//declare Event Delegates
		public delegate void NewClientConnectDelegate(string conID, clsClient client);
		public delegate void ClientDisconnectDelegate(string conID);

		//declare Events
		public event NewClientConnectDelegate NewClient;
		public event ClientDisconnectDelegate ClientDisconnect;

		protected MT.Network.clsBroadcastServer		_Server;
		protected clsClient							_Client;
		protected tsai_converter					_TSAI;
		protected FeaturePointData					_FPData;

		public clsClientCollection(WorkerThread wt): base(wt)
		{
		}

		public void Initialize(MT.Network.clsBroadcastServer Server)
		{
			_Server = Server;
			_Server.NewConnection += new MT.Network.NetClientHandler(_Server_NewConnection);
			_Server.ClosedConnection += new MT.Network.NetEventHandler(_Server_ClosedConnection);
			
		}

		private void _Server_NewConnection(object sender, NetClientEventArgs e)
		{
			//new ClientStatus Class will be created
			_Client = new clsClient(e.NetClient, _Server, m_WorkerThread);
			

			base.Add(e.NetClient.ConnectionID, _Client);

			//Announce new connection by raising event
			if (NewClient!=null){NewClient(e.NetClient.ConnectionID, _Client);}
		}

		private void _Server_ClosedConnection(object sender, NetEventArgs e)
		{
			_Client = null;

			base.Remove(e.ConID);
			//Announce new connection by raising event
			if (ClientDisconnect!=null){ClientDisconnect(e.ConID);}
		}
	}
}
