using System;
using tsai;
using System.Collections;
using MT.Tracker.DataStructures;
using MT.Network;
using MT.Synchronization;
using System.Threading;


namespace MTServer
{
//	public struct strCamera
//	{
//		public string name;
//		public string conID;
//		public clsCamera tsai;
//		public FeaturePointData fpdata;
//		public clsClient client;
//		public int curFrameID;
//		//public FeaturePointFrame curFrame;
//	}

	/// <summary>
	/// Zusammenfassung fr clsTUKData.
	/// </summary>
	public class clsClientSynchronizer
	{
		private clsCameras					_cameras = new clsCameras();
		private clsBroadcastServer			_net;
		private clsFPFrameArray				_FPFrameArray;
		private clsFPFrameArrayContainer	_FPFrameArrayContainer;
		private FeaturePointFrame			_FPFrame;
		private int							_syncSignal = 0;
		private TUKMain						_TUKMain;
		private clsClientCollection			_clientCollection;
		private Thread						_syncThread;
		private bool						_initialize = false;	//Is false as long as tracker has not been initialised
		private bool						_running = false;		//true as long as synchronizer thread is running
		private bool						_stop = false;			//setting this value to true, will gracefully stop the synchronizer
		private int							_framenr = 1;

		public void StopSynchronizer()
		{
			_stop = true;
		}

				
		public clsClientSynchronizer(clsClientCollection clientcollection, clsBroadcastServer net)
		{
			_clientCollection = clientcollection;
			_net = net;
			
			//			_clientCollection.NewClient			+=new MTServer.clsClientCollection.NewClientConnectDelegate(clientcollection_NewClient);
			//			_clientCollection.ClientDisconnect	+=new MTServer.clsClientCollection.ClientDisconnectDelegate(clientcollection_ClientDisconnect);
			_syncThread = new Thread(new ThreadStart(doSynchronize));
			_syncThread.Name = "Synchronizer";	
			_syncThread.IsBackground = true;
			_syncThread.Start();	
			
				
		}

		public void InitializeTUKMain(frm3DView GL3DView)
		{
			_TUKMain = new MTServer.TUKMain(_cameras,GL3DView);
		}

		public void Initialize(clsFPFrameArrayContainer	FPFrameArrayContainer)
		{
			_FPFrameArrayContainer = FPFrameArrayContainer;
		}

		public void StartSynchronizer()
		{
			resetSynchroSignal();
			sendSynchroSignal();

			foreach(clsCamera camera in _cameras.Values)
			{
//				strCamera camera = (strCamera)_cameras.GetByIndex(i);
				camera.CurFrameID = camera.FPData.LastFrame.fid;
			}
			
			_initialize = true;

		}

		/// <summary>
		/// Main Synchronizer Loop
		/// </summary>
		private void doSynchronize()
		{

			_running = true;

			while(!_initialize && !_stop)
			{
				CheckClients();
				Thread.Sleep(200);
			}

			while(!_stop)
			{
				Synchronize();
				Thread.Sleep(0);

			}

			_running = false;
		}

		private void Synchronize()
		{	
			int FrameNr = 0;

			FeaturePointFrame maxSync = new FeaturePointFrame(0,"");
			maxSync.Sync = -1;
			while(maxSync.Sync == -1)
			{
				foreach (clsCamera camera in _cameras.Values)
				{
					_FPFrame = null;

					while((_FPFrame = camera.FPData.GetFrame(camera.CurFrameID)) == null){Thread.Sleep(1);};
					

					if(maxSync.Sync  < _FPFrame.Sync)
					{
						maxSync.Sync = _FPFrame.Sync;
					}

					if (maxSync.Sync == -1) {camera.CurFrameID++;}

				}
			}

			_FPFrameArray = new clsFPFrameArray(_framenr++);
			foreach (clsCamera camera in _cameras.Values)
			{
			
				//Initialize each FPFrame for every Webcam
				
					do
					{
						while((_FPFrame = camera.FPData.GetFrame(camera.CurFrameID)) == null){Thread.Sleep(1);};
						camera.CurFrameID++;

//						if(_FPFrame.Sync < maxSync.Sync)
//                        	Console.WriteLine("Auf dem Client " + camera.Name + " wurde das Frame " + camera.CurFrameID + " gedroppt!");

					} while(_FPFrame.Sync < maxSync.Sync);


//						_FPFrame = camera.FPData.GetFrame(camera.CurFrameID);
//
//						while(_FPFrame.Sync < maxSync.Sync)
//						{
//							
//							while((_FPFrame = camera.FPData.GetFrame(camera.CurFrameID++)) == null){Thread.Sleep(5);};
//							
//						}	

					//Add FPFrame to the _FPFrameArray
					_FPFrameArray.Add(camera.Name,_FPFrame);
					FrameNr = camera.CurFrameID;

			}

			if(FrameNr%30 == 0)
				sendSynchroSignal();

			_FPFrameArrayContainer.Add(_FPFrame.fid,_FPFrameArray);
			_TUKMain.TrackFrame(_FPFrameArray);
			
			
		}
		
		private bool CheckClients()
		{
			int c=0;
			clsClientCollection clientCollectionTemp = _clientCollection;
			while(c <= _clientCollection.Count )
			{
				try
				{
					foreach(clsClient client in clientCollectionTemp.Values)
					{
						if(client.Tsai != null)
						{
							if(_cameras.Count == 0)
							{
								clsCamera					camera;
								camera = new clsCamera(client.Host,client.ConnectionID,new clsCameraDraw(client.Tsai),client.FPData,client,0);
								_cameras.Add(camera.Name, camera);
								//clientCollectionTemp.Remove(client.ConnectionID);
							}
							else
							{
								bool addcamera = true;
								foreach(clsCamera cameratest in _cameras.Values)
								{
									if(cameratest.Name == client.Host)
									{
										addcamera = false;
									}
								}
								if(addcamera)
								{
									clsCamera					camera;
									camera = new clsCamera(client.Host,client.ConnectionID,new clsCameraDraw(client.Tsai),client.FPData,client,0);
									_cameras.Add(camera.Name, camera);
									//clientCollectionTemp.Remove(client.ConnectionID);
								}
							}
							c++;
						}
					}
				}
				catch(System.InvalidOperationException ee)
				{
					Console.WriteLine(ee.ToString());
				}
				Thread.Sleep(200);
			}
			return true;
		}
//		private void clientcollection_NewClient(string conID, clsClient client)
//		{
//			strCamera camera = new strCamera();
//			camera.client	= client;
//			camera.name		= client.Host;
//			camera.conID	= client.ConnectionID;
//			camera.fpdata	= client.FPData;
//
//			camera.tsai		= new clsCamera(client.Tsai);
//			_cameras.Add(camera.name, camera);
//			client.TotalFramesCounted += new MTServer.clsClient.TotalFramesCountedDelegate(FrameID);
//
//		}


//		private void clientcollection_ClientDisconnect(string conID)
//		{
//			foreach (strCamera camera in _cameras.Values)
//			{
//				if ( camera.conID == conID)
//				{
//					_cameras.Remove(camera.name);
//					break;
//				}
//			}
//
//		}

		public TUKMain tukMain
		{
			get{ return _TUKMain;}
			set
			{
				_TUKMain = value;
			}
		}

//		private void checkclients()
//		{
//			foreach (strCamera camera in _cameras.Values)
//			{
//				if ( !camera.client.Tsai.Equals(camera.tsai))
//				{
//					//					camera.tsai=camera.client.Tsai;
//				}
//			}
//		}

		private void resetSynchroSignal()
		{
			_net.Send(new clsNetSyncSignal("",-1));
			Thread.Sleep(1000);
		}

		private void sendSynchroSignal()
		{
				_net.Send(new clsNetSyncSignal("",_syncSignal++)); 
		}

//		private void FrameID(int totalFramesCounted, int frameID)
//		{

//		}

		

		
	}
}
