using System;
using System.Net.Sockets;
using System.Net;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading;


namespace MT.Network
{
	/// <summary>
	/// Starts a clsServer which announces its port and IP over a Broadcast Signal.
	/// </summary>
	public class clsBroadcastServer: clsServer
	{
		private		static IPAddress	_broadcastAddress	= IPAddress.Parse("224.168.100.2");
		private		static int			_broadcastPort		= 2346;

		private		UdpClient	_heartbeatSender	= new UdpClient();
		private		int			_heartbeat;
		private		Thread		_thrHeartbeater;
		private		IPEndPoint	_remoteEP = new IPEndPoint(_broadcastAddress, _broadcastPort); 


		/// <summary>
		/// Starts a clsServer which announces its port and IP over a Broadcast Signal.
		/// </summary>
		/// <param name="Port">Portnumber on which the server listens.</param>
		/// <param name="heartbeat">Heartbeat Interval in Miliseconds between Broadcasts. 0 Indicates a single Broadcast</param>
		public clsBroadcastServer(int Port, int heartbeat): base(Port)
		{
			_heartbeat = heartbeat;

			switch(heartbeat)
			{
				case 0:
					Broadcast();
					break;
				default:
					_thrHeartbeater = new Thread(new ThreadStart(Heartbeat));
					_thrHeartbeater.Name = "BroadcastSender";
					_thrHeartbeater.Start();
					break;
			}

		}

		private void	Heartbeat()
		{
			while(true)
			{
				Broadcast();
				Thread.Sleep(_heartbeat);
			}
		}


		private void	Broadcast()
		{
			//Announce Server per Networkbroadcast
			//Send Portnumber to Multicast Domain of the Clients
			_heartbeatSender.Send(Encoding.ASCII.GetBytes(base._port.ToString()), base._port.ToString().Length, _remoteEP);

		}

//		private void SendBroadcast(string message)
//		{
//
//			IPAddress groupAddress = IPAddress.Parse("224.168.100.2"); 
//			int remotePort = 2346; 
//			IPEndPoint remoteEP = new IPEndPoint(groupAddress, remotePort); 
//			UdpClient server = new UdpClient(); 
//			server.Send(Encoding.ASCII.GetBytes(message), message.Length, remoteEP);
//		}

		#region IDisposable Members

		public override void Dispose()
		{
			_thrHeartbeater.Abort();
			_thrHeartbeater.Join();

			base.Dispose();

		}

		#endregion
	}
}

