using System;
using MT.Tracker.DataStructures;
using System.Runtime.Serialization;
using tsai;
namespace MT.Network
{
	#region "Delegates"
	public		delegate void NetEventHandler(object sender, NetEventArgs e);
	public		delegate void NetMessageHandler(object sender, NetMessageEventArgs e);
	public		delegate void NetObjectHandler(object sender, NetObjectEventArgs e);
	public		delegate void NetClientHandler(object sender, NetClientEventArgs e);
	#endregion

	#region "EventArgs"
	public class NetEventArgs: EventArgs 
	{
		public NetEventArgs(string conID) 
		{
			_conID = conID;
		}
		public	string	ConID
		{
			get { return _conID; }
		}

		private string _conID;

	}   

	public class NetMessageEventArgs: NetEventArgs 
	{
		public NetMessageEventArgs(string conID, string message) : base(conID)
		{
			_message = message;
		}


		public	string	Message
		{
			get { return _message; }
		}

		private string _message;

	}    

	public class NetObjectEventArgs: NetEventArgs 
	{
		public NetObjectEventArgs(clsNet netObject) : base(netObject.ConID)
		{
			_netObject = netObject;
		}


		public	clsNet	NetObject
		{
			get { return _netObject; }
		}

		private clsNet _netObject;

	}   

	public class NetClientEventArgs: NetEventArgs 
	{
		public NetClientEventArgs(clsNetClient netClient) : base(netClient.ConnectionID)
		{
			_netClient = netClient;
		}


		public	clsNetClient	NetClient
		{
			get { return _netClient; }
		}

		private clsNetClient _netClient;

	}   

	#endregion

	#region "NetObjects"
	[Serializable]
	public class clsNet
	{
		private string _ConID ="";

		public string ConID
		{
			get { return _ConID; }
			set { _ConID = value; }
		}

		public clsNet(string ConID)
		{
			_ConID = ConID;
		}
	}

	/// <summary>
	/// clsNetNewConnection is used when requesting a new Connection
	/// it contains a string with the ID which the client wishes to use.
	/// </summary>
	[Serializable]
	public class clsNetNewCon: clsNet
	{
		private string _Hostname;

		public string Hostname
		{
			get { return _Hostname; }
		}

		public clsNetNewCon(string ConID): base(ConID)
		{
		}
		public clsNetNewCon(string ConID, string Hostname): base(ConID)
		{
			_Hostname = Hostname;
		}

	
	}

	/// <summary>
	/// clsNetError is used to transport an ErrorMessage from the server
	/// to the client. This is called whenever a client implied action
	/// lead to an error on the server side.
	/// </summary>
	[Serializable]
	public class clsNetError: clsNet
	{
		private string	_Info;	//Contains Error Message
		private int		_ID;		//Contains ID of the Error
		private clsNet	_Message;
		public string Info
		{
			get { return _Info; }
		}

		public clsNet Message
		{
			get { return _Message; }
		}

		public int	  ID
		{
			get { return _ID; }
		}

		public clsNetError(string ConID, string Info, int ID): base(ConID)
		{
			_Info = Info;
			_ID = ID;
		}
		public clsNetError(string ConID, string Info, int ID, clsNet Message): base(ConID)
		{
			_Info = Info;
			_ID = ID;
			_Message = Message;
		}
	}

	/// <summary>
	/// clsCloseConnection is used when sending a close Connection
	/// request.
	/// </summary>
	[Serializable]
	public class clsNetCloseCon: clsNet
	{
		public clsNetCloseCon(string ConID): base(ConID)
		{
		}
	}

	[Serializable]
	public class clsNetMessage: clsNet
	{
		private string	_Message;	//Contains Error Message
		public string Message
		{
			get { return _Message; }
		}
	
		public clsNetMessage(string ConID, string Message): base(ConID)
		{
			_Message = Message;
		}

	}


	[Serializable]
	public class clsNetFeaturePointFrame: clsNet, IDeserializationCallback
	{
		[NonSerialized]
		private FeaturePointFrame	_frame;	

		private string _strFrame;		//String Converted Frame which will be serialised

		public FeaturePointFrame Frame
		{
			get { return _frame; }
		}
	
		public clsNetFeaturePointFrame(string ConID, FeaturePointFrame frame): base(ConID)
		{
			_frame = frame;
			_strFrame = frame.ToString();
		}

		#region IDeserializationCallback Member

		public void OnDeserialization(object sender)
		{
			_frame = new FeaturePointFrame(_strFrame);
		}

		#endregion
	}

	[Serializable]
	public class clsNetTsaiConverter: clsNet //, IDeserializationCallback
	{
		private tsai_converter	_tsai;	

//		private string _strFrame;		//String Converted Frame which will be serialised

		public tsai_converter tsai_converter
		{
			get { return _tsai; }
		}
	
		public clsNetTsaiConverter(string ConID, tsai_converter tsai): base(ConID)
		{
			_tsai = tsai;
//			_strFrame = frame.ToString();
		}

//		#region IDeserializationCallback Member
//
//		public void OnDeserialization(object sender)
//		{
//			_frame = new FeaturePointFrame(_strFrame);
//		}
//
//		#endregion
	}

	[Serializable]
	public class clsNetSyncSignal: clsNet
	{
		private int	_Message;	//Contains Error Message
		public int Message
		{
			get { return _Message; }
		}
	
		public clsNetSyncSignal(string ConID, int Message): base(ConID)
		{
			_Message = Message;
		}

	}


	/// <summary>
	/// Play Messages contain a boolean value which indicates to the MTClient
	/// if it should enable or disable the FilterGraphController
	/// </summary>
	[Serializable]
	public class clsNetPlay: clsNet
	{
		private bool	_Enable;	//true to enable Player, false to disable
		public bool Enable
		{
			get { return _Enable; }
		}
	
		public clsNetPlay(string ConID, bool Enable): base(ConID)
		{
			_Enable = Enable;
		}

	}


	/// <summary>
	/// Record Messages contain a boolean value which indicates to the MTClient
	/// if it should enable or disable the Capturing Device
	/// </summary>
	[Serializable]
	public class clsNetRecord: clsNet
	{
		private bool	_Enable;	//Contains Error Message
		public bool Enable
		{
			get { return _Enable; }
		}
	
		public clsNetRecord(string ConID, bool Enable): base(ConID)
		{
			_Enable = Enable;
		}

	}
	#endregion

}
