using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Threading;
using MT.Synchronization;

using System.Runtime.Serialization.Formatters.Binary;


namespace MT.Network
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class clsNetClient: SyncInvoker, IDisposable
	{
		//declare Event Delegates
//		public		delegate void NewConnectionDelegate(string conID);
//		public		delegate void ClosedConnectionDelegate(string conID);
//		public		delegate void NewMessageDelegate(string conID, string message);
//		public		delegate void NewMessageObjectDelegate(string conID, clsNet message);
//		public		delegate void NewErrorMessageDelegate(string conID, string errormessage);
//		public		delegate void ConnectingDelegate(string conID);


		//declare Events
		public		event NetEventHandler	NewConnection ;
		public		event NetEventHandler	ClosedConnection;
		public		event NetMessageHandler NewMessage;
		public		event NetObjectHandler	NewMessageObject;
		public		event NetMessageHandler NewErrorMessage;
		public		event NetEventHandler	Connecting;

		//Connection Infos
		protected	string			_host;
		protected	int				_port;
		protected	string			_conID;
		protected	bool			_connected;

		//declare private connection related objects
		private		NetworkStream	_netStream;
		private		TcpClient		_netClient;
		private		Thread			_thrReceiver;

		//used to serialize outgoing and deserialize incoming Objects from the _netStream
		private BinaryFormatter	_bf = new BinaryFormatter();

		#region "Properties"

		public	string	Host
		{
			get { return _host; }
//			set { _host = value; }
		}

		public	int		Port
		{
			get { return _port; }
//			set { _port = value; }
		}

		public	string	ConnectionID
		{
			get { return _conID; }
//			set { _conID = value; }
		}
		#endregion


		/// <summary>
		/// Creates a TCP/IP Connection to the specified Host and Port
		/// </summary>
		/// <param name="conID">Name of the Connection</param>
		/// <param name="Host">Name of the Host</param>
		/// <param name="Port">Port to use on the Host</param>
		public clsNetClient(string conID, string Host, int Port)
		{	
			_conID = conID;
			_host = Host;
			_port = Port;

		}
		protected clsNetClient()
		{
		}
		public clsNetClient(clsNetNewCon NewCon, TcpClient netClient)
		{
			_connected = true;
			_netClient = netClient;
			_conID = NewCon.ConID;
			_host = NewCon.Hostname;

			//Create and start Receiver Thread -> netClient needs to be set!
			StartReceiverThread("Receiver_S_" + _conID);

			Console.WriteLine("clsNetClient: New Connection ({0})","Receiver_S_" + _conID);

		}


		/// <summary>
		/// tries to connect to given server and port, returns true if successful, else false
		/// </summary>
		/// <returns>Returns true when connect call was successful.</returns>
		public virtual bool Connect()
		{
			//Announce ConnectingState
			OnConnecting();

			try
			{
				_netClient = new TcpClient(_host, _port);
			}
			catch(SocketException e)
			{
				Console.WriteLine("Failed to connect to server at {0}. \r\n{1}", _host, e.Message);
				return false;
			}
			catch
			{
				Console.WriteLine("Failed to connect to server at {0}", _host);
				return false;
			}

			//Create and start Receiver Thread -> netClient needs to be set!
			StartReceiverThread("Receiver_C_" + _conID);

			//Inform the Server about the connection ID - it will reply with an extended verified ID
			lock(_netStream)
			{
				_bf.Serialize(_netStream,new clsNetNewCon(_conID, Dns.GetHostName()));
			}

			_connected = true;
			return true;
		}


		/// <summary>
		/// This procedure initalizes the Receiverloop for the _netClient Object.
		/// Do not use, when _netClient is not set!
		/// </summary>
		private void StartReceiverThread(string threadname)
		{	
			if (_netClient!=null)
			{
				//get the Network stream from the TcpClient
				_netStream = _netClient.GetStream();
	
				//Create and start Receiver Thread
				_thrReceiver = new Thread(new ThreadStart(Receiver));
				_thrReceiver.IsBackground = true;
				_thrReceiver.Name = threadname;

				_thrReceiver.Start();
				Thread.Sleep(0);
			}
		}


		private void OnClosedConnection()
		{	
			_connected = false;
			//Announce Connection Closing with event
			Console.WriteLine("The clsNetClient.Receiver Thread (" + _host + ") is shutting down.");
			//if (ClosedConnection !=null) {ClosedConnection(this,new NetEventArgs(_conID)); }
			base.SyncInvoke(ClosedConnection,this, new NetEventArgs(_conID));

		}

		private void OnConnecting()
		{
			//if (Connecting!=null){Connecting(this,new NetEventArgs(_conID));}
			base.SyncInvoke(Connecting,this, new NetEventArgs(_conID));
		}

		private void OnNewConnection()
		{
			try
			{
				base.SyncInvoke(NewConnection,this, new NetEventArgs(_conID));
//				NewConnection(this, new NetEventArgs(msgNewConnection.ConID));
			}
			catch(Exception ex)
			{
				Console.WriteLine("NewConncetion Event threw an Exception: {0}", ex.Message);
			}

		}
		
		private void OnMessage(clsNetMessage msg)
		{
			try
			{
				base.SyncInvoke(NewMessage, this, new NetMessageEventArgs(msg.ConID,msg.Message));
				//NewMessage(this, new NetMessageEventArgs(msg.ConID,msg.Message));
			}
			catch{}
		}

		private void OnMessageObject(clsNet netMessage)
		{
//			if(NewMessageObject!=null)
//			{
			try
			{
				base.SyncInvoke(NewMessageObject, this, new NetObjectEventArgs(netMessage));
				//NewMessageObject(this, new NetObjectEventArgs(netMessage));
			}
			catch (Exception ex)
			{ 
				Console.WriteLine(ex.Message);
			}
//			}

			//						//Server has sent an unimplemented packet
			//						_bf.Serialize(_netStream,new clsNetError(_conID,"An unknown message has been received!",1,netMessage));
			//
			//						try{NewErrorMessage(_conID,"The Server has send a valid but unknown message of Type: " + netMessage.GetType().ToString());}
			//						catch {}

		}


		private void OnErrorMessage(string message)
		{
			base.SyncInvoke(NewErrorMessage,this,new NetMessageEventArgs(_conID,message));
		}


		/// <summary>
		/// Sends the specified Message to the Host. 
		/// </summary>
		/// <param name="message">Message to be sent.</param>
		/// <returns>Returns 'true' if successful, else 'false'.</returns>
		public bool Send(string message)
		{
			if (_connected)
				try
				{
					lock(_netStream)
					{

						_bf.Serialize(_netStream,new clsNetMessage(_conID,message));
					}
					return true;
				}
				catch
				{
					return false;
				}
			return false;
		}
		public bool Send(clsNet message)
		{
			if (_connected)
				try
				{
					lock(_netStream)
					{
						_bf.Serialize(_netStream,message);
					}

					return true;
				}
				catch (Exception ex)
				{
					Console.WriteLine("An Exception occured: {0}", ex.Message);
					return false;
				}
			return false;

		}


		/// <summary>
		///Contains ReceiverLoop which checks the active connection for new messages
		///this method is invoked by a seperate thread. 
		/// </summary>
		private void Receiver()
		{

			clsNet			netMessage=null;
			clsNetMessage	msg;
			clsNetNewCon	msgNewConnection;
			clsNetError		msgError;
			clsNetCloseCon	msgCloseConnection;

			try
			{

				//Enter Messageloop
				do
				{
					Thread.Sleep(0);
					// Load serialized Object from the net-stream
					netMessage = _bf.Deserialize(_netStream) as clsNet;

					//if Object is not derived from clsNet, return an error
					//to the client and close connection
					if (netMessage == null)
					{
						lock(_netStream)
						{
							_bf.Serialize(_netStream,new clsNetError(_conID,"An invalid Message has been received!",0));
						}
						
						try
						{
							OnErrorMessage("An invalid Message has been received!");
							//NewErrorMessage(this,new NetMessageEventArgs(_conID,"An invalid Message has been received!"));
						
						}
						catch {}

					}


					if ((msg = netMessage as clsNetMessage)!=null) 
					{	//Announce Message by raising event / The EventCode is packaged in a try
						//clause to prevent catching exceptions from the eventreceivers
						OnMessage(msg);
					}
					else if ((msgError = netMessage as clsNetError) !=null)
					{
						//						foreach(Delegate d in NewErrorMessage.GetInvocationList())
						//						{
						//							d.DynamicInvoke
						//						}
						try
						{
							Console.WriteLine("Server meldete Fehler: {0}", msgError.Info);
							
							base.SyncInvoke(NewErrorMessage, this, new NetMessageEventArgs(msgError.ConID,msgError.Info));
//							if (NewErrorMessage!=null)
//							{
//								NewErrorMessage(this,new NetMessageEventArgs(msgError.ConID,msgError.Info));
//							}
						}
						catch{}
					}
					else if ((msgNewConnection = netMessage as clsNetNewCon) != null)
					{
						if(msgNewConnection.ConID != _conID)
						{
							_conID = msgNewConnection.ConID;
						}
						//Announce new Connection by raising event
						OnNewConnection();
					
					}

					else if ((msgCloseConnection = netMessage as clsNetCloseCon) != null)
					{
						netMessage = null;
					}	
					else
					{
						OnMessageObject(netMessage);
					}

				} while (netMessage!=null);
				//If message is null, the connection has been closed

			}
			catch (System.Net.Sockets.SocketException)
			{	
					Console.WriteLine("Die Verbindung wurde vom Server beendet");
			}
			catch (Exception ex)				  
			{
					Console.WriteLine("An exception occured: {0}",ex.Message);
			}
			finally
			{			
				OnClosedConnection();
				//				throw new Exception("The clsNetClient.Receiver Thread (" + Thread.CurrentThread.Name + ")threw an exception. The Network Connection has been shut down!");
			}

		}


		public void CloseConnection()
		{

			if (_netStream!=null)
			{
				try
				{
					//Send Connection Close Message
					lock(_netStream)
					{
						_bf.Serialize(_netStream,new clsNetCloseCon(_conID));
					}
					_netStream.Close();
				}
				catch {}
            }

			//Shutdown Receiver Thread
			//_thrReceiver.Suspend();
			if(_thrReceiver!=null)
			{
				_thrReceiver.Abort();
				_thrReceiver.Join();
			}
		}


		#region "IDisposable Members"

		public virtual void Dispose()
		{
			//Shutdown Receiver Thread
			//_thrReceiver.Suspend();
			if(_thrReceiver!=null)
			{
				_thrReceiver.Abort();
				_thrReceiver.Join();
			}

			if(_netStream!=null)
				_netStream.Close();

			if(_netClient!=null)
				_netClient.Close();

			_netStream = null;
			_netClient = null;
		}
		#endregion

	}

}

