using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections;

namespace MT.Network
{
	public struct strConInfo
	{
		public IPAddress	IP;
		public int			Port;
		public bool			Set;

		public override string ToString()
		{
			return IP.ToString();
		}

	}

	/// <summary>
	/// A Client that listens for a server signals on a given MulticastGroup/Port and
	/// allows to choose to which server you would like to connect.
	/// Use Enable/Disable to activate or deactivate this listening mode.
	/// Disable will also disconnect the Client from an already running connection.
	/// </summary>
	public class clsNetSemiAutoClient: clsNetClient
	{
		public delegate void ListeningDelegate(string conID);
		public event ListeningDelegate Listening;

		public delegate void ServerListDelegate(strConInfo[] ConInfos);
		public event ServerListDelegate ServerList;

		private static readonly IPAddress	_GroupAddress	= IPAddress.Parse("224.168.100.2");
		private static readonly int			_GroupPort		= 2346;

		private					Thread		_thrReceiver;
		private					SortedList	_ConInfos = new SortedList();
		private					strConInfo	_ConInfo;

		private					bool		_Enabled = false;
		private					bool		_Listening = false;


		public void SetConInfo(strConInfo ConInfo)
		{
			_ConInfo = ConInfo;
//			Connect(_ConInfo);
		}


		public clsNetSemiAutoClient(string ConID)
		{
			base._conID = ConID;
			base.ClosedConnection += new NetEventHandler(_base_ClosedConnection);

		}

		/// <summary>
		/// Activates the Client Listener
		/// </summary>
		public void Enable()
		{
			lock(this)
			{
				if (_Enabled!=true)
				{
					StartListener();
					_Enabled = true;
				}
			}


		}

		/// <summary>
		/// Deactivates the Listener and/or disconnects
		/// the Client if already connected.
		/// </summary>
		public void Disable()
		{
			_Enabled = false;
		}

		public override bool Connect()
		{
			Enable();

			return true;
		}

		private bool Connect(strConInfo ConInfo)
		{
			if (ConInfo.Set)
			{
				//Configure and start Client with Broadcast Parameters
				base._host = ConInfo.IP.ToString();
				base._port = ConInfo.Port;
				base.Connect();	

				return true;
			}

			return false;

		}

		/// <summary>
		/// Starts up the Listening Thread
		/// </summary>
		private void StartListener()
		{
			_thrReceiver = new Thread(new ThreadStart(Listen));
			_thrReceiver.IsBackground = true;
			_thrReceiver.Start();
		}

		private void Listen() 
		{
			lock(this)
			{	
				if (_Listening == false)
				//Announce Listening mode
				_Listening = true;
				else
					return;
			}

			UdpClient client= null;

			try
			{

				while (client==null)
					try
					{	//Try to create UdpClient on _GroupPort
						client= new UdpClient(_GroupPort);
						Thread.Sleep(500);
					}
					catch{}

				client.JoinMulticastGroup(_GroupAddress);
				IPEndPoint iep = new IPEndPoint(IPAddress.Any, 0);
			
				//Announce ListeningState
				OnListening();

				byte[] data;

				while(_ConInfo.Set = false || !base._connected)
				{
					data = client.Receive(ref iep);
			
					//Store Received ConInfo in our pool
					strConInfo ConInfo = new strConInfo();
					ConInfo.IP = iep.Address;
					ConInfo.Port = int.Parse(Encoding.ASCII.GetString(data));
					ConInfo.Set = true;
			
					if (_ConInfo.IP !=null && _ConInfo.IP.ToString() == ConInfo.IP.ToString())
					{
						Connect(_ConInfo);
						Thread.Sleep(100);
					}
					else if (!_ConInfos.Contains(ConInfo.IP.ToString()))
					{
						_ConInfos.Add(ConInfo.IP.ToString(), ConInfo);
					}

					OnServerList();
				}

			}
			catch{}
			finally
			{
				//Shutdown Broadcast Receiver Client
				client.DropMulticastGroup(_GroupAddress);
				client.Close();

				OnServerList();

				//Clear ConnectionInfos List whenever aborting Listening
				_ConInfos.Clear();

				lock(this)
				{	//Announce Listening mode
					_Listening = false;
				}

			}
	}


		private void _base_ClosedConnection(object sender, NetEventArgs e)
		{
			if (_Enabled)
				StartListener();
		}

		#region "EventRaisers"

		private void OnListening()
		{
			if (Listening!=null){Listening(base._conID);}
		}

		private void OnServerList()
		{
			if (ServerList!=null) 
			{
				int i =0;
				strConInfo[] ConInfos = new strConInfo[_ConInfos.Count];
				foreach (strConInfo ConInfo in _ConInfos.Values)
				{
					ConInfos[i++] = ConInfo;
				}

				ServerList(ConInfos);
			}
		}

		#endregion

		#region IDisposable Members

		public override void Dispose()
		{
			if (_thrReceiver!=null)
			{
				//				_thrReceiver.Suspend();
				//				_thrReceiver.Interrupt();

				_thrReceiver.Abort();
				//				_thrReceiver.Join();

				_thrReceiver = null;
			}

			base.Dispose();

		}

		#endregion
	
	}
}
