using System;
using System.Collections;
using System.Diagnostics;
using System.ComponentModel;
using System.Threading;
using System.Security.Permissions;

namespace MT.Synchronization
{
	/// <summary>
	/// Zusammenfassung fr SynchronizerSortedList.
	/// </summary>
	[SecurityPermission(SecurityAction.Demand,ControlThread = true)]
	public class SynchronizerSortedList: SortedList, ISynchronizeInvoke, IDisposable
	{
		protected WorkerThread m_WorkerThread;

		internal WorkerThread workerThread
		{
			get { return m_WorkerThread; }
		}

		public bool InvokeRequired 
		{
			get
			{
				bool res = Thread.CurrentThread.GetHashCode() == m_WorkerThread.GetHashCode();
				return  ! res;
			}
		}
		public IAsyncResult BeginInvoke(Delegate method, object[] args)
		{
			WorkItem result = new WorkItem(null,method,args);
			m_WorkerThread.QueueWorkItem(result);
			return result;
		}
		public object EndInvoke(IAsyncResult result)
		{
			result.AsyncWaitHandle.WaitOne();
			WorkItem workItem = (WorkItem)result;
			return  workItem.MethodReturnedValue;
		}
		public object Invoke(Delegate method, object[] args)
		{
			IAsyncResult asyncResult;
			asyncResult = BeginInvoke(method,args);
			return EndInvoke(asyncResult);
		}
		public SynchronizerSortedList()
		{ 
			m_WorkerThread = new WorkerThread(this);
		}
		protected SynchronizerSortedList(WorkerThread workerThread)
		{
			m_WorkerThread = workerThread;
		}
		~SynchronizerSortedList()
		{
		}
		public void Dispose()
		{
			if (m_WorkerThread.Synchronizer == this)
				m_WorkerThread.Kill();
		}
	}
}
